%******************************************************************************;
%* Copyright (C) 2004 by SAS Institute Inc., Cary, NC 27512-8000               ;
%*                                                                             ;
%* Name:     Waetl                                                             ;
%*                                                                             ;
%* Purpose:  Initialize or Build and Update the Web Analytics Web Mart         ;
%*                                                                             ;
%* Parameters:                                                                 ;
%*                                                                             ;
%*            name:  Specifies the name of the Web Analytics and e-Data ETL    ;
%*                   web mart.                                                 ;
%*                                                                             ;
%*                   When program=warehouse                                    ;
%*                   If name is specified it is used to identify               ;
%*                   the ETL data store location in the SASUSER.WBINIT data set;
%*                   Once the data store has been identified, a search for     ;
%*                   the default detail data set(i.e. weblog_detail_1)         ;
%*                   is initiated. If successful in locating detail the        ;
%*                   warehouse build process is started. Name is also used to  ;
%*                   identify the Web Analytics Web Mart(i.e. swamart) location;
%*                   in the SASUSER.WAINIT data set.                           ;
%*                                                                             ;
%*                   When program=initialize                                   ;
%*                   name is stored in the SASUSER.WBINIT for the purpose of   ;
%*                   identifying the e-Data ETL  data store. It is also stored ;
%*                   in the SASUSER.WAINIT data set for the purpose of         ;
%*                   identifying the Web Analytics Web Mart(i.e. swamart).     ;
%*                                                                             ;
%*            data_store: Specifies the location of the e-Data ETL  web mart.  ;
%*                                                                             ;
%*                        When program=warehouse                               ;
%*                        This parameter is used to specify the location of the;
%*                        e-Data ETL  web mart. Then the default detail data set;
%*                        is searched. If the default data set is located then ;
%*                        then the warehouse build process is started.         ;
%*                        data store overrides the specification name but does ;
%*                        not override the specification of detail.            ;
%*                                                                             ;
%*             detail: Specifies the detail data set to be processed.          ;
%*                                                                             ;
%*                        When program=warehouse                               ;
%*                        This value is used to locate the detail data set.    ;
%*                        If it exists then proceed with the warehouse build.  ;
%*                                                                             ;
%*             temp_store: Specifies a location for temporary data sets.       ;
%*                                                                             ;
%*                         If this location is not specified or does not exist ;
%*                         then the work library location is used for temporary;
%*                         files and data sets.                                ;
%*                                                                             ;
%******************************************************************************;

%macro waetl(name=
            ,data_store=
            ,temp_store=
            ,detail=
            ,swamart=
            ,program=);

%******************************************************************************;
%* SECTION: Preamble                                                           ;
%*  Compile the Web Analytics Utility Macros                                   ;
%******************************************************************************;
%wautils;


%******************************************************************************;
%* SECTION: I - Preliminary                                                    ;
%******************************************************************************;

%******************************************************************************;
%* MODULE: I_A - Environment Configuration                                     ;
%* SUPPORT:saskxs                                                              ;
%* Declare and Initialize the necessary macro variables and determine the OS.  ;
%******************************************************************************;
%local macname;
%let   macname=&sysmacroname;

%let module=I_A;
%let macname=&macname._&module;

%******************************************************************************;
%* Create global macro variables                                               ;
%******************************************************************************;

%global _wab_rc_;
%let _wab_rc_=0;

%******************************************************************************;
%* Create and initialize local macro variables                                 ;
%******************************************************************************;

%local wab_rc;

%local wab_note
       wab_warning
       wab_error
       wab_iter;

%let   wab_note    = %nrstr(NOTE:(&macname) %left(%sysfunc(time(),time.)));
%let   wab_warning = %nrstr(WARNING:(&macname) %left(%sysfunc(time(),time.)));
%let   wab_error   = %nrstr(ERROR:(&macname) %left(%sysfunc(time(),time.)));

%local wab_program;

%let wab_program=;

%******************************************************************************;
%* If the program parameter is specified then compress and lowcase its value   ;
%******************************************************************************;

%if %quote(&program) ne %then %do;

   %let wab_program=%sysfunc(compress(%sysfunc(lowcase(&program))));

%end;

%******************************************************************************;
%* If the swa_mart or data_store parameters are specified then translate quotes;
%* to blanks                                                                   ;
%******************************************************************************;


%if %quote(&swamart) ne %then %do;

   %let swamart=%sysfunc(strip(%sysfunc(translate(&swamart," ","'"))));
   %let swamart=%sysfunc(strip(%sysfunc(translate(&swamart,' ','"'))));

%end;

%if %quote(&data_store) ne %then %do;

   %let data_store=%sysfunc(strip(%sysfunc(translate(&data_store," ","'"))));
   %let data_store=%sysfunc(strip(%sysfunc(translate(&data_store,' ','"'))));

%end;

%******************************************************************************;
%* The following macro variables set the 1, 7, and 30 day minimum support/count;
%* for the number of sessions which view a sequences of pages/URLS in the      ;
%* pathing summaries. As well as the maximum number of pages a sequence can    ;
%* have.                                                                       ;
%******************************************************************************;

%local wab_r1_support
       wab_r7_support
       wab_r30_support
       wab_itemset_size;

%let   wab_r1_support=10;
%let   wab_r7_support=40;
%let   wab_r30_support=100;
%let   wab_itemset_size=7;

%******************************************************************************;
%* Create local macro variable for the data set that contains the most recent  ;
%* date.                                                                       ;
%******************************************************************************;

%local wab_datecheck_dsn;

%******************************************************************************;
%* Determine the Operating System                                              ;
%******************************************************************************;
dm 'af catalog=SASHELP.WAGUI.WAINI.SCL';
%Current_OS;
%if &_wab_rc_ ne 0 %then %goto ERREXIT;

%******************************************************************************;
%* Declare and Initialize the summary metadata set name and proc summary file  ;
%* count.                                                                      ;
%******************************************************************************;

%local wab_metadsn
       wab_summary_count;

%let wab_metadsn= ;
%let wab_summary_count=0;

%******************************************************************************;
%* Declare and Initialize number of intervals to keep in summary data.         ;
%* Declare and Initialize the number of days to keep in the daily visitor      ;
%* and pathing tables.                                                         ;
%******************************************************************************;

%local wab_days_in_history
       wab_weeks_in_history
       wab_months_in_history
       wab_quarters_in_history
       wab_years_in_history
       wab_visitor_days_in_history
       wab_pathing_days_in_history;

%let wab_days_in_history=90;
%let wab_weeks_in_history=52;
%let wab_months_in_history=36;
%let wab_quarters_in_history=12;
%let wab_years_in_history=3;

%let wab_visitor_days_in_history=60;
%let wab_pathing_days_in_history=30;


%******************************************************************************;
%* Initialize the minimum date to keep in the daily visitor table to missing.  ;
%* This will prevent any subsetting of the table in case it is not initialized ;
%* to keep the most current number of days in history as specified by          ;
%* wab_visitor_days_in_history.                                                ;
%******************************************************************************;

data _null_;
  call symput("_wab_visitor_min_date_",".z");
run;

%******************************************************************************;
%* Declare and Initialize number of data sets to keep in web mart.             ;
%*    NOTE: these assignments are over-ridden by the values in the WACONFIG    ;
%*          metadata.                                                          ;
%******************************************************************************;
%local wab_num_detail_data_sets
       wab_num_session_data_sets;

%let wab_num_detail_data_sets=7;
%let wab_num_session_data_sets=14;

%******************************************************************************;
%* Declare and Initialize the number or URLs to store in the drop down lists   ;
%* of the funnel form.                                                         ;
%******************************************************************************;

%******************************************************************************;
%* Declare and initialize the sample sizes for Automatic Segmentation training ;
%* partition and validation partition. Also specify the number of segments     ;
%* to be output for the Automatic Segmentation runs.                           ;
%******************************************************************************;
%local wab_autoseg_train_samp_pct
       wab_autoseg_valid_samp_pct
       wab_autoseg_num_segments;

%let wab_autoseg_train_samp_pct=0.6;
%let wab_autoseg_valid_samp_pct=%sysevalf(1.0-&wab_autoseg_train_samp_pct);
%let wab_autoseg_num_segments=4;

%******************************************************************************;
%* Declare and initialize the parameters which determine the construction of   ;
%* the decision trees used to generate the segments. These include the maximum ;
%* depth of the splitting rules used to generate the segments, the significance;
%* level for the identification of the splitting rules, and the number of      ;
%* observations required for a splitting rule to be considered.                ;
%******************************************************************************;
%local wab_autoseg_alpha
       wab_autoseg_minworth
       wab_autoseg_maxdepth
       wab_autoseg_leafsize;


%let wab_autoseg_alpha=0.20;
%let wab_autoseg_minworth=0.0;
%let wab_autoseg_maxdepth=10;
%let wab_autoseg_leafsize=;

%******************************************************************************;
%* Initialize global and local return codes.                                   ;
%******************************************************************************;
%if %quote(&_wab_rc_)= %then %do;
  %let _wab_rc_=0;
%end;

%if %quote(&wab_rc)= %then %do;
  %let wab_rc=0;
%end;

%******************************************************************************;
%* Save system options to be restored upon exit                                ;
%******************************************************************************;
%let notes_setting    = %sysfunc(getoption(NOTES));
%let source_setting   = %sysfunc(getoption(SOURCE));
%let fmterr_setting   = %sysfunc(getoption(FMTERR));
%let fmtsearch_setting= %sysfunc(getoption(FMTSEARCH));
%let source2_setting  = %sysfunc(getoption(SOURCE2));
%let mprint_setting   = %sysfunc(getoption(MPRINT));
%let compress_setting = %sysfunc(getoption(COMPRESS));
%let ovp_setting      = %sysfunc(getoption(OVP));
%let obs_setting      = %sysfunc(getoption(OBS));

%if x&obs_setting eq x9223372036854775807 %then
   %let obs_setting = MAX;

options compress=yes noovp obs=max nofmterr;

%No_Notes;          %* Turn off notes to start;
%Maybe_Notes;       %* Enable notes if not Quiet;

%******************************************************************************;
%* Print out version information and memory option information.                ;
%******************************************************************************;
%put %unquote(&wab_note) %qcmpres(Starting Web Analytics running on
                                SAS &sysvlong.);
%put %unquote(&wab_note) %qcmpres(
     Options memsize %sysfunc(getoption(memsize))%str(,)
             sumsize %sysfunc(getoption(sumsize))%str(,)
             sortsize %sysfunc(getoption(sortsize))).;

%******************************************************************************;
%* If the location and name of the Web Analytics Web Mart are not specified    ;
%* fail.                                                                       ;
%******************************************************************************;

%if %quote(&swamart)= and %quote(&name)= %then %do;
    %put %unquote(&wab_error) Both the %str(%")swamart%str(%") %QCMPRES(
         parameter)and the %str(%")name%str(%")%QCMPRES(
         parameter) were not provided.  One of them must be specified %QCMPRES(
         in) a call to WAETL.;
    %goto ERREXIT;
%end;


%******************************************************************************;
%* MODULE: I_B - Web Analytics Web Mart Initialization                         ;
%* SUPPORT:saskxs                                                              ;
%* Create the directory structure for the Web Analytics Web Mart.              ;
%* Copy necessary files to the configuration directory.                        ;
%******************************************************************************;
%let module=I_B;
%let macname=%scan(&macname,1,_)_&module;
%let insert_name= ;

%if %quote(&wab_program)=initialize %then %do;
   %***************************************************************************;
   %* If the Web Analytics Web Mart location was not specified then fail.      ;
   %***************************************************************************;
   %if %quote(&swamart)= %then %do;
      %put %unquote(&wab_error) The %str(%")swamart%str(%") %QCMPRES(
           parameter) was not provided and must be specified.;
      %goto ERREXIT;
   %end;


   %***************************************************************************;
   %* Get the name of the Web Analytics Web Mart either from the name parameter;
   %* or from the name of the deepest subfolder in the swamart path.           ;
   %***************************************************************************;
   data _null_;
      %if %quote(&name)= %then %do;
         call symput("_wab_swaname_"
                    ,reverse(scan(reverse(strip("&swamart"))
                                 ,1
                                 ,"&separator")));
      %end;
      %else %do;
         call symput("_wab_swaname_"
                    ,strip("&name"));
      %end;
   run;


   %***************************************************************************;
   %* If the name of the Web Analytics Web Mart contains only whitespace then  ;
   %* issue an error and exit Web Analytics.                                   ;
   %***************************************************************************;
   %if %quote(&_wab_swaname_)= %then %do;
      %put %unquote(&wab_error) The name for the Web Analytics %QCMPRES(
           could) Mart contains only whitespace. Please %QCMPRES(
           specify) the %str(%")name%str(%") parameter.;
      %goto ERREXIT;
   %end;


   %***************************************************************************;
   %*   SASUSER.WBINIT                                                         ;
   %***************************************************************************;
   %* If it does NOT exist:                                                    ;
   %*    - create it                                                           ;
   %*    - seed it with the current name, web mart location, and description.  ;
   %***************************************************************************;
   %if %sysfunc(exist(sasuser.wbinit))=0 %then %do;
      data sasuser.wbinit;

         attrib  Name length=$32    label="Webmart Name"
                 Desc length=$200   label="Description"
                 Perm length=$200   label="Webmart Location"
                 Flag length=3      label="Internal use"
                 ;

         Name="&_wab_swaname_";
         Desc="Created by SAS Web-Analytics";
         Perm="&swamart.&separator.e-data-etl";
         Flag=1;

      run;
   %end;

   %***************************************************************************;
   %* If it does exist:                                                        ;
   %*    - if it cannot be opened fail WAETL                                   ;
   %*    - if it does not have any obs, create one and seed it with the        ;
   %*      current name, web mart location, and description                    ;
   %*    - else if _WAB_SWANAME_ is in the NAME field for one of the obs:      ;
   %*       - if SWAMART path exists fail WAETL                                ;
   %*       - if SWAMART path equals PERM path issue warning                   ;
   %*       - create an obs and seed it with the SWANAME (+ suffix),           ;
   %*          web mart location, and description                              ;
   %*    - else (_WAB_SWANAME_ is NOT in the NAME field for one of the obs):   ;
   %*       - create DATA_STORE to identify the path                           ;
   %*       - if the path exists fail WAETL                                    ;
   %*       - else create an obs and seed it with the current name, web mart   ;
   %*         location, and description                                        ;
   %***************************************************************************;
   %else %do;
      %* - if it cannot be opened fail WAETL                                   ;
      %locktest(dset=sasuser.wbinit);
      %if &lock_sw ne 0 %then %do;
          %put %unquote(&wab_error) There was a failure opening the %QCMPRES(
               SASUSER.WBINIT) dataset.;
          %goto ERREXIT;
      %end;
      %else %if %get_observation_count(indsn=sasuser.wbinit)=0 %then %do;
         %*    - if it does not have any obs, create one and seed it with the  ;
         %*      current name, web mart location, and description              ;
          data sasuser.wbinit;
             attrib  Name length=$32    label="Webmart Name"
                     Desc length=$200   label="Description"
                     Perm length=$200   label="Webmart Location"
                     Flag length=3      label="Internal use"
                    ;
             Name="&_wab_swaname_";
             Desc="Created by SAS Web-Analytics";
             Perm="&swamart.&separator.e-data-etl";
             Flag=1;
          run;
      %end;
      %else %do;
         %*   - if _WAB_SWANAME_ is in the NAME field for one of the obs:      ;
         data subset;
            set sasuser.wbinit
               (where=(upcase(name) eq upcase("&_wab_swaname_")));
         run;
         data _null_;
            set sasuser.wbinit end=eof;
            if upcase(name) eq upcase("&_wab_swaname_") then do;
               numobs+1;
               call symput('data_store',strip(perm));
            end;
            if eof then do;
               call symput('numobs',strip(put(numobs,5.)));
            end;
         run;

         %if &numobs ge 1 %then %do;
            %*   &_WAB_SWANAME_ is already in WBINIT -- check if PERM exists   ;
            %let new_path=&swamart&separator.e-data-etl;
            %if %sysfunc(fileexist(&new_path)) %then %do;
               %* if the path identified by &NEW_PATH exists fail WAETL        ;
               %put %unquote(&wab_error) The SWAMART location that you %QCMPRES(
                  have) specified already exists.  To initialize a Web %QCMPRES(
                  mart), please specify another location.;
               %goto ERREXIT;
            %end;
            %if  %quote(%upcase(&new_path)) eq
                 %quote(%upcase(&data_store))    %then %do;
               %*   if SWAMART path equals PERM path issue WARNING             ;
               %put %unquote(&wab_warning) SASUSER.WBINIT already %QCMPRES(
                  has) an entry for &_wab_swaname_ at %QCMPRES(
                  &data_store).  Another entry will be specified for %QCMPRES(
                  this) location.;
            %end;
            %*   identify the next usable suffix for &_WAB_SWANAME_            ;
            data suffix_list (keep=name);
               set sasuser.wbinit;
               name=upcase(name);
            run;
            proc sort data=suffix_list
                         (where=(index(upcase(name),upcase("&_wab_swaname_"))));
               by name;
            run;
            %if %get_observation_count(indsn=suffix_list) eq 0 %then %do;
               %let insert_name=&_wab_swaname_;
            %end;
            %else %do;
               data _null_;
                  set suffix_list end=eof;
                  length new_suffix $ 2;
                  if eof then do;
                     new_suffix=tranwrd(strip(upcase(name))
                                       ,strip(upcase("&_wab_swaname_"))
                                       ,' '
                                       );
                     new_suffix=strip(new_suffix);
                     if new_suffix eq ' ' or new_suffix ge 'A' then
                        new_suffix='1';
                     else
                        new_suffix=put((input(new_suffix,2.)+1),2.);
                     call symput('insert_name'
                                ,strip("&_wab_swaname_") || strip(new_suffix)
                                );
                  end;
               run;
               %put %unquote(&wab_warning) SASUSER.WBINIT already %QCMPRES(
                  has) an entry for &_wab_swaname_ -- its e-Data-ETL %QCMPRES(
                  root) is at &data_store..  WAETL will refer to %QCMPRES(
                  your) new Web mart as &insert_name to distinguish it %QCMPRES(
                  from) its predecessors.;
            %end;
            %*   else create an obs and seed it with the curr name (+ suffix), ;
            %*   web mart location, and description                            ;
            data sasuser.wbinit;
               set sasuser.wbinit end=eof;
               output;
               if eof then do;
                  Name="&insert_name";
                  Desc="Created by SAS Web Analytics";
                  Perm="&swamart.&separator.e-data-etl";
                  Flag=1;
                  output;
               end;
            run;
         %end;
         %else %do;
            %*   &_WAB_SWANAME_ is NOT in the NAME field for one of the obs:   ;
            %let data_store=&swamart.&separator.e-data-etl;
            %if %sysfunc(fileexist(&data_store)) %then %do;
               %* if the path exists fail WAETL                                ;
               %put %unquote(&wab_error) SASUSER.WBINIT already has an %QCMPRES(
                  entry) for the location you have specified %QCMPRES(
                  --) &data_store..  To initialize a Web mart, please %QCMPRES(
                  specify) another location.;
               %goto ERREXIT;
            %end;
            %*  else create an obs and seed it with the current name, web mart ;
            %*  location, and description                                      ;
            data sasuser.wbinit;
               set sasuser.wbinit end=eof;
               output;
               if eof then do;
                  Name="&_wab_swaname_";
                  Desc="Created by SAS Web Analytics";
                  Perm="&swamart.&separator.e-data-etl";
                  Flag=1;
                  output;
               end;
            run;
         %end;
      %end;
   %end;


   %***************************************************************************;
   %*   SASUSER.WAINIT                                                         ;
   %***************************************************************************;
   %* If it does NOT exist:                                                    ;
   %*    - create it                                                           ;
   %*    - seed it with the current name, web mart location, and description.  ;
   %***************************************************************************;
   %if %sysfunc(exist(sasuser.wainit))=0 %then %do;
      data sasuser.wainit;

         attrib  Name length=$32    label="Webmart Name"
                 Desc length=$200   label="Description"
                 Perm length=$200   label="Webmart Location"
                 Flag length=3      label="Internal use"
                 ;

         Name="&_wab_swaname_";
         Desc="Created by SAS Web-Analytics";
         Perm="&swamart.&separator.e-data-etl";
         Flag=1;

      run;
   %end;

   %***************************************************************************;
   %* If it does exist:                                                        ;
   %*    - if it cannot be opened fail WAETL                                   ;
   %*    - if it does not have any obs, create one and seed it with the        ;
   %*      current name, web mart location, and description                    ;
   %*    - else if _WAB_SWANAME_ is in the NAME field for one of the obs:      ;
   %*       - if SWAMART path exists fail WAETL                                ;
   %*       - if SWAMART path equals PERM path issue warning                   ;
   %*       - create an obs and seed it with the SWANAME (+ suffix),           ;
   %*          web mart location, and description                              ;
   %*    - else (_WAB_SWANAME_ is NOT in the NAME field for one of the obs):   ;
   %*       - create DATA_STORE to identify the path                           ;
   %*       - if the path exists fail WAETL                                    ;
   %*       - else create an obs and seed it with the current name, web mart   ;
   %*         location, and description                                        ;
   %***************************************************************************;
   %else %do;
      %* - if it cannot be opened fail WAETL                                   ;
      %locktest(dset=sasuser.wainit);
      %if &lock_sw ne 0 %then %do;
          %put %unquote(&wab_error) There was a failure opening the %QCMPRES(
               SASUSER.WAINIT) dataset.;
          %goto ERREXIT;
      %end;
      %else %if %get_observation_count(indsn=sasuser.wainit)=0 %then %do;
         %*    - if it does not have any obs, create one and seed it with the  ;
         %*      current name, web mart location, and description              ;
          data sasuser.wainit;
             attrib  Name length=$32    label="Webmart Name"
                     Desc length=$200   label="Description"
                     Perm length=$200   label="Webmart Location"
                     Flag length=3      label="Internal use"
                    ;
             Name="&_wab_swaname_";
             Desc="Created by SAS Web-Analytics";
             Perm="&swamart.&separator.e-data-etl";
             Flag=1;
          run;
      %end;
      %else %do;
         %*   - if _WAB_SWANAME_ is in the NAME field for one of the obs:      ;
         data subset;
            set sasuser.wainit
               (where=(upcase(name) eq upcase("&_wab_swaname_")));
         run;
         data _null_;
            set sasuser.wainit end=eof;
            if upcase(name) eq upcase("&_wab_swaname_") then do;
               numobs+1;
               call symput('data_store',strip(perm));
            end;
            if eof then do;
               call symput('numobs',strip(put(numobs,5.)));
            end;
         run;

         %if &numobs ge 1 %then %do;
            %*   &_WAB_SWANAME_ is already in WAINIT -- check if PERM exists   ;
            %let new_path=&swamart&separator.e-data-etl;
            %if %sysfunc(fileexist(&new_path)) %then %do;
               %* if the path identified by &NEW_PATH exists fail WAETL        ;
               %put %unquote(&wab_error) The SWAMART location that you %QCMPRES(
                  have) specified already exists.  To initialize a Web %QCMPRES(
                  mart), please specify another location.;
               %goto ERREXIT;
            %end;
            %if  %quote(%upcase(&new_path)) eq
                 %quote(%upcase(&data_store))    %then %do;
               %*   if SWAMART path equals PERM path issue WARNING             ;
               %put %unquote(&wab_warning) SASUSER.WAINIT already %QCMPRES(
                  has) an entry for &_wab_swaname_ at %QCMPRES(
                  &data_store).  Another entry will be specified for %QCMPRES(
                  this) location.;
            %end;
            %*   identify the next usable suffix for &_WAB_SWANAME_            ;
            data suffix_list (keep=name);
               set sasuser.wainit;
               name=upcase(name);
            run;
            proc sort data=suffix_list
                         (where=(index(upcase(name),upcase("&_wab_swaname_"))));
               by name;
            run;
            %if %get_observation_count(indsn=suffix_list) eq 0 %then %do;
               %let insert_name=&_wab_swaname_;
            %end;
            %else %do;
               data _null_;
                  set suffix_list end=eof;
                  length new_suffix $ 2;
                  if eof then do;
                     new_suffix=tranwrd(strip(upcase(name))
                                       ,strip(upcase("&_wab_swaname_"))
                                       ,' '
                                       );
                     new_suffix=strip(new_suffix);
                     if new_suffix eq ' ' or new_suffix ge 'A' then
                        new_suffix='1';
                     else
                        new_suffix=put((input(new_suffix,2.)+1),2.);
                     call symput('insert_name'
                                ,strip("&_wab_swaname_") || strip(new_suffix)
                                );
                  end;
               run;
               %put %unquote(&wab_warning) SASUSER.WAINIT already %QCMPRES(
                  has) an entry for &_wab_swaname_ -- its e-Data-ETL %QCMPRES(
                  root) is at &data_store..  WAETL will refer to %QCMPRES(
                  your) new Web mart as &insert_name to distinguish it %QCMPRES(
                  from) its predecessors.;
            %end;
            %*   else create an obs and seed it with the curr name (+ suffix), ;
            %*   web mart location, and description                            ;
            data sasuser.wainit;
               set sasuser.wainit end=eof;
               output;
               if eof then do;
                  Name="&insert_name";
                  Desc="Created by SAS Web Analytics";
                  Perm="&swamart.&separator.e-data-etl";
                  Flag=1;
                  output;
               end;
            run;
         %end;
         %else %do;
            %*   &_WAB_SWANAME_ is NOT in the NAME field for one of the obs:   ;
            %let data_store=&swamart.&separator.e-data-etl;
            %if %sysfunc(fileexist(&data_store)) %then %do;
               %* if the path exists fail WAETL                                ;
               %put %unquote(&wab_error) SASUSER.WAINIT already has an %QCMPRES(
                  entry) for the location you have specified %QCMPRES(
                  --) &data_store..  To initialize a Web mart, please %QCMPRES(
                  specify) another location.;
               %goto ERREXIT;
            %end;
            %*  else create an obs and seed it with the current name, web mart ;
            %*  location, and description                                      ;
            data sasuser.wainit;
               set sasuser.wainit end=eof;
               output;
               if eof then do;
                  Name="&_wab_swaname_";
                  Desc="Created by SAS Web Analytics";
                  Perm="&swamart.&separator.e-data-etl";
                  Flag=1;
                  output;
               end;
            run;
         %end;
      %end;
   %end;


   %***************************************************************************;
   %* Build the new location                                                   ;
   %***************************************************************************;
   %make_directory(dirname=&swamart
                ,retcode=wab_rc);
   %if &wab_rc ne 0 %then
      %goto ERREXIT;


   %***************************************************************************;
   %* Create local macro variables to contain directory structure.             ;
   %***************************************************************************;
   %local wab_config_dir
          wab_logs_dir
          wab_sas_dir
          wab_data_dir
          wab_dated_dir
          wab_detl_dir
          wab_etl_dir
          wab_doc_dir
          wab_stp_dir
          wab_smy_dir
         ;

   %let wab_config_dir=&swamart.&separator.config;
   %let wab_logs_dir=&swamart.&separator.logs;
   %let wab_sas_dir=&swamart.&separator.sas;
   %let wab_data_dir=&swamart.&separator.data;
   %let wab_dated_dir=&swamart.&separator.data&separator.dated;
   %let wab_detl_dir=&swamart.&separator.data&separator.detail;
   %let wab_etl_dir=&swamart.&separator.e-data-etl;
   %let wab_doc_dir=&swamart.&separator.data&separator.documents;
   %let wab_stp_dir=&swamart.&separator.data&separator.stp;
   %let wab_smy_dir=&swamart.&separator.data&separator.summary;


   %***************************************************************************;
   %* Create the directory structure for Web Analytics.                        ;
   %***************************************************************************;
   %if not %sysfunc(fileexist(&wab_config_dir)) %then %do;
      %make_directory(dirname=&wab_config_dir
                     ,retcode=wab_rc);
   %end;
   %if &wab_rc ne 0 %then
      %goto ERREXIT;

   %if not %sysfunc(fileexist(&wab_logs_dir)) %then %do;
      %make_directory(dirname=&wab_logs_dir
                      ,retcode=wab_rc);
   %end;
   %if &wab_rc ne 0 %then
      %goto ERREXIT;

   %if not %sysfunc(fileexist(&wab_sas_dir)) %then %do;
      %make_directory(dirname=&wab_sas_dir
                     ,retcode=wab_rc);
   %end;
   %if &wab_rc ne 0 %then
      %goto ERREXIT;

    %if not %sysfunc(fileexist(&wab_data_dir)) %then %do;
       %make_directory(dirname=&wab_data_dir
                      ,retcode=wab_rc);
    %end;
   %if &wab_rc ne 0 %then
      %goto ERREXIT;

   %if not %sysfunc(fileexist(&wab_dated_dir)) %then %do;
      %make_directory(dirname=&wab_dated_dir
                     ,retcode=wab_rc);
   %end;
   %if &wab_rc ne 0 %then
      %goto ERREXIT;

   %if not %sysfunc(fileexist(&wab_detl_dir)) %then %do;
      %make_directory(dirname=&wab_detl_dir
                     ,retcode=wab_rc);
   %end;
   %if &wab_rc ne 0 %then
      %goto ERREXIT;

   %if not %sysfunc(fileexist(&wab_etl_dir)) %then %do;
      %make_directory(dirname=&wab_etl_dir
                     ,retcode=wab_rc);
   %end;
   %if &wab_rc ne 0 %then
      %goto ERREXIT;

   %if not %sysfunc(fileexist(&wab_doc_dir)) %then %do;
      %make_directory(dirname=&wab_doc_dir
                     ,retcode=wab_rc);
   %end;
   %if &wab_rc ne 0 %then
      %goto ERREXIT;

   %if not %sysfunc(fileexist(&wab_stp_dir)) %then %do;
      %make_directory(dirname=&wab_stp_dir
                     ,retcode=wab_rc);
   %end;
   %if &wab_rc ne 0 %then
      %goto ERREXIT;

   %if not %sysfunc(fileexist(&wab_smy_dir)) %then %do;
      %make_directory(dirname=&wab_smy_dir
                     ,retcode=wab_rc);
   %end;
   %if &wab_rc ne 0 %then
      %goto ERREXIT;


   %***************************************************************************;
   %* Assign the Web Mart config library and move the necessary metadata to    ;
   %* the config library location. If the copy fails then exit with error.     ;
   %***************************************************************************;
   libname config "&wab_config_dir";

   proc copy in=sashelp out=config memtype=data;
      select waadmsum
             waautinp
             waautosg
             waconfig
             wadshbrd
             wadshinp
             waengvar
             wafunnel
             wapath
             waresrce
             wascrcrd
             wascrinp
             watbldsc
            ;
   run;

   %if &syserr gt 4 %then %do;
      %put %unquote(&wab_error) WAADMSUM, WAAUTINP, WAAUTOSG,  %QCMPRES(
           WACONFIG), WADSHBRD, WADSHINP, WAENGVAR, WAFUNNEL, %QCMPRES(
            WAPATH), WARESRCE, WASCRCRD, WASCRINP, or %QCMPRES(
           WATBLDSC) could not be copied into CONFIG.;
      %goto ERREXIT;
   %end;


   %***************************************************************************;
   %* Initialize the e-Data ETL environment.                                   ;
   %***************************************************************************;
   %webhound(data_store=&swamart.&separator.e-data-etl);


   %***************************************************************************;
   %* Create the standard .sas programs in the Web marts \sas sub-directory    ;
   %***************************************************************************;
   %wasasdir(retcode=wab_rc);
   %if &wab_rc ne 0 %then
      %goto ERREXIT;


   %***************************************************************************;
   %* Everything OK go to exit.                                                ;
   %***************************************************************************;
   %if &insert_name gt  %then %do;
      %let rpt_name=%quote(&insert_name);
   %end;
   %else %do;
      %let rpt_name=%quote(&_wab_swaname_);
   %end;
   %put %unquote(&wab_note) The Web Analytics Web Mart %QCMPRES(
        &rpt_name) at location &swamart was %QCMPRES(
        successfully) initialized. Please configure e-Data ETL.;

   %goto EXIT;

%end;

%***************************************************************************;
%* If an invalid value for the program parameter was passed then issue an   ;
%* error and fail. Valid values are initialize and warehouse.               ;
%***************************************************************************;
%else %if %quote(&wab_program) ne warehouse %then %do;

   %put %unquote(&wab_error) You must specify a program parameter %QCMPRES(
        for %str(%)WAETL) to execute with the PROGRAM parameter.;
   %goto ERREXIT;

%end;

%******************************************************************************;
%*Program=warehouse, proceed to check for swamart location and detail          ;
%*location.                                                                    ;
%******************************************************************************;

%if %quote(&swamart) ne %then %do;

    %if not %sysfunc(fileexist(&swamart)) %then %do;
       %put %unquote(&wab_error) The &swamart location for the Web %QCMPRES(
            Analytics) Web Mart does not exist.;
       %goto ERREXIT;
    %end;

%end;

%******************************************************************************;
%* If swamart was not specified and name was specified then look in the Web    ;
%* Analytics listing data set for the location and verify its existence.       ;
%******************************************************************************;

%if %quote(&swamart)= and %quote(&name) ne %then %do;

   %if %sysfunc(exist(sasuser.wainit))=0 %then %do;
      %put %unquote(&wab_error) The SASUSER.WAINIT data set does not exist.;
      %goto ERREXIT;
   %end;

   %locktest(dset=sasuser.wainit);
   %if &lock_sw ne 0 %then %do;
      %put %unquote(&wab_error) There was a failure opening %QCMPRES(
           the) SASUSER.WAINIT dataset.;
      %goto ERREXIT;
   %end;

   %if %get_observation_count(indsn=sasuser.wainit)=0  %then %do;
      %put %unquote(&wab_error) The SASUSER.WAINIT data set %QCMPRES(
           contains) 0 observations.;
      %goto ERREXIT;
   %end;

   %***************************************************************************;
   %* Identify the swamart via the name and store it.                          ;
   %***************************************************************************;

   proc sql noprint;

      select perm into :swamart
         from sasuser.wainit
            where strip(upcase(name)) eq strip(upcase("&name")) ;

   quit;

   %***************************************************************************;
   %* Make sure there are no duplicate Data Stores.                            ;
   %***************************************************************************;

   %if &sqlobs gt 1 %then %do;
      %put %unquote(&wab_error) Found multiple entries for environment %QCMPRES(
           &name) in SASUSER.WAINIT.;
      %goto ERREXIT;
   %end;

   %***************************************************************************;
   %* Make sure Data Store was set to something.                               ;
   %***************************************************************************;

   %else %if &sqlobs eq 0 %then %do;
      %put %unquote(&wab_error) Cannot locate entry in %QCMPRES(
           SASUSER.WAINIT) for Web Mart: &name.;
      %put %unquote(&wab_error) Check the value of the %QCMPRES(
           %str(%"))name%str(%") parameter.;
      %goto ERREXIT;
   %end;

   %***************************************************************************;
   %* Make sure the Data Store was set to something.                           ;
   %***************************************************************************;

   %if %quote(&swamart)= %then %do;
      %put %unquote(&wab_error) Cannot locate a value for the %QCMPRES(
           %str(%")data_store%str(%")) parameter in %QCMPRES(
           SASUSER.WAINIT) for environment: &name.;
      %put %unquote(&wab_error) Check the value of the %QCMPRES(
           %str(%"))name%str(%") parameter.;
      %goto ERREXIT;
   %end;

%end;

%let swamart=%sysfunc(strip(&swamart));

%******************************************************************************;
%* Create local macro variables to contain directory structure.                ;                                                                       ;
%******************************************************************************;

%local wab_config_dir wab_logs_dir wab_sas_dir wab_data_dir wab_dated_dir
       wab_detl_dir wab_etl_dir wab_doc_dir wab_stp_dir wab_smy_dir;

%let wab_config_dir=&swamart.&separator.config;
%let wab_data_dir=&swamart.&separator.data;
%let wab_dated_dir=&swamart.&separator.data&separator.dated;
%let wab_detl_dir=&swamart.&separator.data&separator.detail;
%let wab_doc_dir=&swamart.&separator.data&separator.documents;
%let wab_stp_dir=&swamart.&separator.data&separator.stp;
%let wab_smy_dir=&swamart.&separator.data&separator.summary;
%let wab_etl_dir=&swamart.&separator.e-data-etl;
%let wab_logs_dir=&swamart.&separator.logs;
%let wab_sas_dir=&swamart.&separator.sas;

%******************************************************************************;
%*Verify the existence of the Web Analtics directory structure.                ;
%* The following locations must exist:                                         ;
%*                                                                             ;
%*          swamart--+                                                         ;
%*                   |                                                         ;
%*                   +config                                                   ;
%*                   |                                                         ;
%*                   |                                                         ;
%*                   +data--+                                                  ;
%*                          |                                                  ;
%*                          +dated                                             ;
%*                          |                                                  ;
%*                          +detail                                            ;
%*                          |                                                  ;
%*                          +documents                                         ;
%*                          |                                                  ;
%*                          +stp                                               ;
%*                          |                                                  ;
%*                          +summary                                           ;
%*                   |                                                         ;
%*                   |                                                         ;
%*                   +e-data-etl                                               ;
%*                   |                                                         ;
%*                   |                                                         ;
%*                   +logs                                                     ;
%*                   |                                                         ;
%*                   |                                                         ;
%*                   +sas                                                      ;
%*                                                                             ;
%******************************************************************************;


%if not %sysfunc(fileexist(&wab_config_dir)) %then %do;
   %put %unquote(&wab_error) The &wab_config_dir location for the Web %QCMPRES(
            Analytics) Web Mart configuration data directory does not exist.;
   %goto ERREXIT;
%end;

%if not %sysfunc(fileexist(&wab_data_dir)) %then %do;
   %put %unquote(&wab_error) The &wab_data_dir location for the Web %QCMPRES(
            Analytics) Web Mart data directory does not exist.;
   %goto ERREXIT;
%end;

%if not %sysfunc(fileexist(&wab_dated_dir)) %then %do;
   %put %unquote(&wab_error) The &wab_dated_dir location for the Web %QCMPRES(
            Analytics) Web Mart date partitioned data directory does not exist.;
   %goto ERREXIT;
%end;

%if not %sysfunc(fileexist(&wab_detl_dir)) %then %do;
   %put %unquote(&wab_error) The &wab_detl_dir location for the Web %QCMPRES(
            Analytics) Web Mart detail data directory does not exist.;
   %goto ERREXIT;
%end;

%if not %sysfunc(fileexist(&wab_doc_dir)) %then %do;
   %put %unquote(&wab_error) The &wab_doc_dir location for the Web %QCMPRES(
            Analytics) Web Mart documents data directory does not exist.;
   %goto ERREXIT;
%end;

%if not %sysfunc(fileexist(&wab_stp_dir)) %then %do;
   %put %unquote(&wab_error) The &wab_stp_dir location for the Web %QCMPRES(
            Analytics) Web Mart STP data directory does not exist.;
   %goto ERREXIT;
%end;

%if not %sysfunc(fileexist(&wab_smy_dir)) %then %do;
   %put %unquote(&wab_error) The &wab_smy_dir location for the Web %QCMPRES(
            Analytics) Web Mart summary data directory does not exist.;
   %goto ERREXIT;
%end;

%if not %sysfunc(fileexist(&wab_logs_dir)) %then %do;
   %put %unquote(&wab_error) The &wab_logs_dir location for the Web %QCMPRES(
            Analytics) Web Mart sas logs directory does not exist.;
   %goto ERREXIT;
%end;

%if not %sysfunc(fileexist(&wab_sas_dir)) %then %do;
   %put %unquote(&wab_error) The &wab_sas_dir location for the Web %QCMPRES(
            Analytics) Web Mart sas programs directory does not exist.;
   %goto ERREXIT;
%end;

%*****************************************************************************;
%* Assign the necessary data libraries for Web Analytics.                     ;
%*****************************************************************************;
libname config "&wab_config_dir";
libname dated  "&wab_dated_dir";
libname summary "&wab_smy_dir";
libname wadetail "&wab_detl_dir";
libname document "&wab_doc_dir";


%******************************************************************************;
%* Initialize macro parameters from web analytics config data set.             ;
%******************************************************************************;
%if %sysfunc(exist(config.waconfig)) ne 0 %then %do;
   %locktest(dset=config.waconfig);
   %if &lock_sw eq 0 %then %do;
      %if %get_observation_count(indsn=config.waconfig) gt 0 %then %do;
         %let bad_week_start=1;
         data _null_;
            set config.waconfig end=eof;
            retain wab_pathing_days_in_history
                   wab_num_session_data_sets;
            length week_shift $ 2;

            if upcase(parameter_name) eq 'WEEK_START' then do;
               call symput('bad_week_start','0');
               if      upcase(value) eq 'SUNDAY'    then week_shift='.1';
               else if upcase(value) eq 'MONDAY'    then week_shift='.2';
               else if upcase(value) eq 'TUESDAY'   then week_shift='.3';
               else if upcase(value) eq 'WEDNESDAY' then week_shift='.4';
               else if upcase(value) eq 'THURSDAY'  then week_shift='.5';
               else if upcase(value) eq 'FRIDAY'    then week_shift='.6';
               else if upcase(value) eq 'SATURDAY'  then week_shift='.7';
               else do;
                  call symput('bad_week_start',strip(value));
                  stop;
               end;
               call symput('wab_week_shift',strip(week_shift));
            end;
            else do;
               call symput(parameter_name,strip(value));
            end;

            if upcase(parameter_name) eq 'WAB_PATHING_DAYS_IN_HISTORY' then
               wab_pathing_days_in_history=value;
            else if upcase(parameter_name) eq 'WAB_NUM_SESSION_DATA_SETS' then
               wab_num_session_data_sets=value;
            if eof then do;
               %***************************************************************;
               %*   when WAB_NUM_SESSION_DATA_SETS is less than                ;
               %*      WAB_PATHING_DAYS_IN_HISTORY, the drill-down report for  ;
               %*      the Interactive Funnel will fail.  as the arguments are ;
               %*      assigned for the WACONFIG-based macro parameters, this  ;
               %*      condition is detected and expressed in the DATED_MSG    ;
               %*      macro (blank if the condition does not exist).  also,   ;
               %*      the WACONFIG value for WAB_NUM_SESSION_DATA_SETS is     ;
               %*      set to the value of WAB_PATHING_DAYS_IN_HISTORY.        ;
               %*                                                              ;
               %*   the message is not output until III_A - the aging section. ;
               %***************************************************************;
               if wab_num_session_data_sets   lt
                  wab_pathing_days_in_history    then do;
                     call symput('dated_msg'
                                ,"&wab_warning WACONFIG has a value of "      ||
                                 strip(wab_num_session_data_sets)             ||
                                 ' for the WAB_NUM_SESSION_DATA_SETS'         ||
                                 ' parameter.  This value must be >= '        ||
                                 strip(wab_pathing_days_in_history)           ||
                                 ', the value for the'                        ||
                                 ' WAB_PATHING_DAYS_IN_HISTORY parameter or'  ||
                                 ' else the Interactive Funnel drill-down'    ||
                                 ' report will fail.  WAETL is using '        ||
                                 strip(wab_pathing_days_in_history)           ||
                                 ' as the WAB_NUM_SESSION_DATA_SETS parameter'||
                                 ' to control the retention of DATED.SESSION' ||
                                 '_yyyymmdd data sets.'
                                );
                     call symput('wab_num_session_data_sets'
                             ,strip(wab_pathing_days_in_history)
                             );
               end;
               else
                  call symput('dated_msg',' ');
            end;
         run;
         %if &bad_week_start eq 1 %then %do;
            %put %unquote(&wab_note) CONFIG.WACONFIG does not have %QCMPRES(
                 an) entry for the WEEK_START parameter.  The default %QCMPRES(
                 entry) -- Sunday -- is being used.;
            %let wab_week_shift=.1;
         %end;
         %else %if &bad_week_start ne 0 %then %do;
            %put %unquote(&wab_error) CONFIG.WACONFIG has an invalid %QCMPRES(
                 value) -- &bad_week_start -- for the WEEK_START %QCMPRES(
                 parameter).  Please assign a standard English-%QCMPRES(
                 language) day-of-week to the WEEK_START parameter.;
            %goto ERREXIT;
         %end;
      %end;
   %end;
%end;
%******************************************************************************;
%*   Create SUMMARY.WADTSORT                                                   ;
%******************************************************************************;
%wametdef(def_dsn=wadtsort
         ,retcode=wab_rc
         );


%*****************************************************************************;
%* Ensure that either the location of, ETL web mart name of or the actual     ;
%* detail data set has been specified.                                        ;
%*****************************************************************************;
%if %quote(&data_store)= and %quote(&name)= and %quote(&detail)= %then %do;


   %if %sysfunc(fileexist(&wab_etl_dir)) %then %do;

      libname detail "&wab_etl_dir.&separator.detail";

      %let detail=detail.weblog_detail_1;

      %***************************************************************************;
      %* If the ETL detail data set has been specified then ensure its            ;
      %* existence, a positive number of records, and that it can be opened.      ;
      %***************************************************************************;

      %if %sysfunc(exist(&detail))=0 %then %do;
         %put %unquote(&wab_error) The detail data set does not exist.;
         %goto ERREXIT;
      %end;

      %locktest(dset=&detail);
      %if &lock_sw ne 0 %then %do;
         %put %unquote(&wab_error) There was a failure opening the detail dataset.;
         %goto ERREXIT;
      %end;

      %if %get_observation_count(indsn=&detail)=0  %then %do;
         %put %unquote(&wab_error) The detail data set contains 0 observations.;
         %goto ERREXIT;
      %end;

   %end;

   %else %do;

      %put %unquote(&wab_error) The %str(%")data_store%str(%") %QCMPRES(
           parameter)%str(,) the %str(%")name%str(%")%str(,) or the %QCMPRES(
           %str(%")detail%str(%")) parameter of the %nrstr(%%)waetl %QCMPRES(
           macro) must be specified.;
      %goto ERREXIT;

   %end;

%end;

%******************************************************************************;
%* If the ETL detail data set has been specified then ensure its               ;
%* existence, a positive number of records, and that it can be opened.         ;
%******************************************************************************;

%if %quote(&detail) ne %then %do;

   %if %sysfunc(exist(&detail))=0 %then %do;
      %put %unquote(&wab_error) The detail data set does not exist.;
      %goto ERREXIT;
   %end;

   %locktest(dset=&detail);
   %if &lock_sw ne 0 %then %do;
      %put %unquote(&wab_error) There was a failure opening the detail dataset.;
      %goto ERREXIT;
   %end;

   %if %get_observation_count(indsn=&detail)=0  %then %do;
      %put %unquote(&wab_error) The detail data set contains 0 observations.;
      %goto ERREXIT;
   %end;

%end;

%******************************************************************************;
%* If the ETL data store has been specified then ensure its existence.         ;
%******************************************************************************;

%if %quote(&detail)= and %quote(&data_store) ne %then %do;

   %if not %sysfunc(fileexist(&data_store.&separator.detail)) %then %do;
      %put %unquote(&wab_error) The location of the ETL detail data %QCMPRES(
      set) does not exist.;
      %goto ERREXIT;
   %end;

   libname detail "&data_store.&separator.detail";
   libname control "&data_store.&separator.control";

   %let detail=detail.weblog_detail_1;

   %if %sysfunc(exist(&detail))=0 %then %do;
      %put %unquote(&wab_error) The detail data set does not exist.;
      %goto ERREXIT;
   %end;

   %locktest(dset=&detail);
   %if &lock_sw ne 0 %then %do;
      %put %unquote(&wab_error) There was a failure opening the detail dataset.;
      %goto ERREXIT;
   %end;

   %if %get_observation_count(indsn=&detail)=0  %then %do;
      %put %unquote(&wab_error) The detail data set contains 0 observations.;
      %goto ERREXIT;
   %end;

%end;

%******************************************************************************;
%* If the web mart name has been specified then ensure its existence in        ;
%* the e-Data ETL  listing.                                                     ;
%******************************************************************************;

%if %quote(&detail)= and %quote(&data_store)= and %quote(&name) ne %then %do;

   %if %sysfunc(exist(sasuser.wbinit))=0 %then %do;
      %put %unquote(&wab_error) The sasuser.wbinit data set does not exist.;
      %goto ERREXIT;
   %end;

   %locktest(dset=sasuser.wbinit);
   %if &lock_sw ne 0 %then %do;
      %put %unquote(&wab_error)
         There was a failure opening the sasuser.wbinit dataset.;
      %goto ERREXIT;
   %end;

   %if %get_observation_count(indsn=sasuser.wbinit)=0  %then %do;
      %put %unquote(&wab_error)
         The sasuser.wbinit data set contains 0 observations.;
      %goto ERREXIT;
   %end;

   %***************************************************************************;
   %* Identify the data store via the name and store it.                       ;
   %***************************************************************************;

   proc sql noprint;

      select perm into :data_store
         from sasuser.wbinit
            where strip(upcase(name)) eq strip(upcase("&name"));

   quit;

   %***************************************************************************;
   %* Make sure there are no duplicate Data Stores.                            ;
   %***************************************************************************;

   %if &sqlobs gt 1 %then %do;
      %put %unquote(&wab_error) Found multiple entries for environment %QCMPRES(
           &name) in sasuser.wbinit.;
      %goto ERREXIT;
   %end;

   %***************************************************************************;
   %* Make sure Data Store was set to something.                               ;
   %***************************************************************************;

   %else %if &sqlobs eq 0 %then %do;
      %put %unquote(&wab_error) Cannot locate entry in %QCMPRES(
           SASUSER.WBINIT) for environment: &name.;
      %put %unquote(&wab_error) Check the value of the %QCMPRES(
           %str(%"))name%str(%") parameter.;
      %goto ERREXIT;
   %end;

   %***************************************************************************;
   %* Make sure the Data Store was set to something.                           ;
   %***************************************************************************;

   %if %quote(&data_store)= %then %do;
      %put %unquote(&wab_error) Cannot locate a value for the %QCMPRES(
           %str(%")data_store%str(%")) parameter in %QCMPRES(
           SASUSER.WBINIT) for environment: &name.;
      %put %unquote(&wab_error) Check the value of the %QCMPRES(
           %str(%"))name%str(%") parameter.;
      %goto ERREXIT;
   %end;

   %let data_store=%sysfunc(strip(&data_store));

   %if not %sysfunc(fileexist(&data_store.&separator.detail)) %then %do;
      %put %unquote(&wab_error) The location of the ETL detail data %QCMPRES(
      set) does not exist.;
      %goto ERREXIT;
   %end;

   libname detail "&data_store.&separator.detail";
   libname control "&data_store.&separator.control";

   %let detail=detail.weblog_detail_1;

   %***************************************************************************;
   %* If the ETL detail data set has been specified then ensure its            ;
   %* existence, a positive number of records, and that it can be opened.      ;
   %***************************************************************************;

   %if %sysfunc(exist(&detail))=0 %then %do;
      %put %unquote(&wab_error) The detail data set does not exist.;
      %goto ERREXIT;
   %end;

   %locktest(dset=&detail);
   %if &lock_sw ne 0 %then %do;
      %put %unquote(&wab_error) There was a failure opening the detail dataset.;
      %goto ERREXIT;
   %end;

   %if %get_observation_count(indsn=&detail)=0  %then %do;
      %put %unquote(&wab_error) The detail data set contains 0 observations.;
      %goto ERREXIT;
   %end;

%end;

%******************************************************************************;
%* Initialize the temporary library to the physical location specified by      ;
%* temp_store. If temp_store does not exist then let the temporary library be  ;
%* to the location /swawork under work. Write warning out to the log if        ;
%* physical location does not exist.                                           ;
%******************************************************************************;

%if %quote(&temp_store)= %then %do;

   %let temp_store=%sysfunc(pathname(work,L));
   %let temp_store=&temp_store.&separator.swawork;

   %if not
   %sysfunc(fileexist(&temp_store))
   %then %do;

      %make_directory(dirname=&temp_store
                     ,retcode=wab_rc);

         %end;

   %if &wab_rc ne 0 %then
   %goto ERREXIT;

   libname worklib "&temp_store";
   %let temp_lib=worklib;

%end;

%else %do;

   %***************************************************************************;
   %* If the temporary location exists then ensure that it does not equal any  ;
   %* of the permanent data locations in the web mart. If it passes this test  ;
   %* Set the temporary library to worklib.                                    ;
   %***************************************************************************;

   %if %sysfunc(fileexist(&temp_store)) %then %do;

      %if %sysfunc(strip(%sysfunc(lowcase("&temp_store"))))
          =%sysfunc(strip(%sysfunc(lowcase("&wab_dated_dir")))) %then %do;
          %put %unquote(&wab_error) The temporary location for data %QCMPRES(
               sets) cannot be the same as the permanent location for %QCMPRES(
               date) partitioned data sets.;
          %goto ERREXIT;
      %end;

      %if %sysfunc(strip(%sysfunc(lowcase("&temp_store"))))
          =%sysfunc(strip(%sysfunc(lowcase("&wab_detl_dir")))) %then %do;
          %put %unquote(&wab_error) The temporary location for data %QCMPRES(
               sets) cannot be the same as the permanent location for %QCMPRES(
               detail) data sets.;
          %goto ERREXIT;
      %end;

      %if %sysfunc(strip(%sysfunc(lowcase("&temp_store"))))
          =%sysfunc(strip(%sysfunc(lowcase("&wab_doc_dir")))) %then %do;
          %put %unquote(&wab_error) The temporary location for data %QCMPRES(
               sets) cannot be the same as the permanent location for %QCMPRES(
               document) data sets.;
          %goto ERREXIT;
      %end;

      %if %sysfunc(strip(%sysfunc(lowcase("&temp_store"))))
          =%sysfunc(strip(%sysfunc(lowcase("&wab_stp_dir")))) %then %do;
          %put %unquote(&wab_error) The temporary location for data %QCMPRES(
               sets) cannot be the same as the permanent location for %QCMPRES(
               stored process) data sets.;
          %goto ERREXIT;
      %end;

      %if %sysfunc(strip(%sysfunc(lowcase("&temp_store"))))
          =%sysfunc(strip(%sysfunc(lowcase("&wab_smy_dir")))) %then %do;
          %put %unquote(&wab_error) The temporary location for data %QCMPRES(
               sets) cannot be the same as the permanent summary %QCMPRES(
               location) for data sets.;
          %goto ERREXIT;
      %end;

      libname worklib "&temp_store";
      %let temp_lib=worklib;

   %end;

   %***************************************************************************;
   %* If the temporary location does not exist then set the temporary library  ;
   %* to the location /swawork under work.                                     ;
   %***************************************************************************;

   %else %do;

      %let temp_store=%sysfunc(pathname(work,L));
      %let temp_store=&temp_store.&separator.swawork;

      %if not
      %sysfunc(fileexist(&temp_store))
      %then %do;

         %make_directory(dirname=&temp_store
                        ,retcode=wab_rc);

      %end;

      %if &wab_rc ne 0 %then %do;
         %put %unquote(&wab_error) The temporary storage location was %QCMPRES(
              not) specified and the default one could not be created.;
         %goto ERREXIT;
      %end;

      libname worklib "&temp_store";
      %let temp_lib=worklib;

   %end;

%end;

%******************************************************************************;
%* Create a list of data sets in the temporary library and use proc datasets   ;
%* to delete them all.                                                         ;
%******************************************************************************;

proc sql;
   create table &temp_lib..temp_tables as
      select memname
      from dictionary.tables
           where upcase(compress(libname))=upcase(compress("&temp_lib"));
run;
quit;

%******************************************************************************;
%* If the temp directory contains at least one SAS data set then call PROC     ;
%* DATASETS to delete the temp directorys contents                             ;
%******************************************************************************;


%if &sqlobs gt 0 %then %do;

   data _null_;
      set &temp_lib..temp_tables end=eof;
      call symput(compress("_wab_temp_memname_"||put(_n_,7.)),memname);
      if eof then call symput("_wab_tot_tables_",put(_n_,7.));
   run;

   proc datasets library=&temp_lib nolist;
   delete %do wab_iter=1 %to &_wab_tot_tables_;
              &&_wab_temp_memname_&wab_iter
          %end;
              temp_tables;
   run;
   quit;

%end;

%******************************************************************************;
%* Delete all files in the temporary location.                                 ;
%******************************************************************************;
%if %upcase(&sysscp) eq WIN or
    %upcase(&sysscp) eq DNTHOST %then %do;
   %execcmd(&delete_all "&temp_store&separator.*.*"
           ,_rc=wab_rc);
%end;
%else %do;
   %execcmd(&delete_all "&temp_store&separator.*"
           ,_rc=wab_rc);
%end;


%if &syserr gt 4 %then %do;
   %put %unquote(&wab_error) Legacy files in the temporary %QCMPRES(
        location:) &temp_store could not be deleted. Please check permissions.;
   %goto ERREXIT;
%end;

%******************************************************************************;
%* Set the format search path to look in the following library order:          ;
%* 1. WORK                                                                     ;
%* 2. LIBRARY                                                                  ;
%* 1. CONFIG                                                                   ;
%* 2. CONTROL                                                                  ;
%******************************************************************************;

options fmtsearch=(work library config control);

%******************************************************************************;
%* MODULE: I_C - Date Checking                                                 ;
%* SUPPORT:saskxs                                                              ;
%* Check Dates in ETL, Web Analytics Detail , and Web Analytics Session        ;
%* Data.                                                                       ;
%******************************************************************************;
%let module=I_C;
%let macname=%scan(&macname,1,_)_&module;

proc sql;

  %****************************************************************************;
  %* Get the uniques dates from ETL detail                                     ;
  %* to process.                                                               ;
  %****************************************************************************;

  create table &temp_lib..etl_detail_dates as
     select distinct date
        from &detail
           where date gt .z;

  %****************************************************************************;
  %* Get the list of the dates from Web Analytics kept detail.                 ;
  %****************************************************************************;

  create table &temp_lib..detail_library_contents as
     select memname,
            input(scan(memname,2,"_"),YYMMDD10.) as date
        from dictionary.tables
           where upcase(libname)="WADETAIL"
              and prxmatch('/^DETAIL_\d{8}$/', upcase(left(trim(memname))));

  %****************************************************************************;
  %* Create the list of kept session data sets from                            ;
  %* the dated directory.                                                      ;
  %****************************************************************************;

  create table &temp_lib..dated_library_contents as
     select memname,
            input(scan(memname,2,"_"),YYMMDD10.) as date
       from dictionary.tables
          where upcase(libname)="DATED"
             and prxmatch('/^SESSION_\d{8}$/', upcase(left(trim(memname))));

  %****************************************************************************;
  %* Get the list of dates for data which needs to be updated.                 ;
  %* If the date exists in ETL detail, the kept detail, and                    ;
  %* in sessions data then the summaries will be updated.                      ;
  %****************************************************************************;

  create table &temp_lib..update_dates as
     select date
        from  (select date from &temp_lib..etl_detail_dates
           where date in (select distinct date
                          from &temp_lib..detail_library_contents))
              where date in (select distinct date
                             from &temp_lib..dated_library_contents);

  %****************************************************************************;
  %* Create the list of dates for which data needs to be created.              ;
  %* If the date exists in ETL detail and does not exist in kept detail, and   ;
  %* and does not exist in sessions data. New rows in the summaries will be    ;
  %* created for these dates.                                                  ;
  %****************************************************************************;

  create table &temp_lib..new_dates as
     select date
        from  (select date from &temp_lib..etl_detail_dates
           where date not in (select distinct date
                              from &temp_lib..detail_library_contents))
              where date not in (select distinct date
                                 from &temp_lib..dated_library_contents);

  %****************************************************************************;
  %* Create the list of dates for which we cannot create data.                 ;
  %* If we do not have detail in the warehouse for a day we have processed     ;
  %* in the summaries then we cannot recreate session correctly because we     ;
  %* do not have data for that day.                                            ;
  %****************************************************************************;

  create table &temp_lib..invalid_dates as
     select date
        from  (select date from &temp_lib..etl_detail_dates
           where date not in (select distinct date
                              from &temp_lib..detail_library_contents))
              where date in (select distinct date
                             from &temp_lib..dated_library_contents);

quit;

%******************************************************************************;
%* Ensure that the data set containing dates processed in the ETL detail       ;
%* data set exists, has observations, and can be opened.                       ;
%******************************************************************************;

%if %sysfunc(exist(&temp_lib..etl_detail_dates))=0 %then %do;
   %put %unquote(&wab_error) The &temp_lib..etl_detail_dates data set %QCMPRES(
        with) dates from etl does not exist.;
   %goto ERREXIT;
%end;

%locktest(dset=&temp_lib..etl_detail_dates);
%if &lock_sw ne 0 %then %do;
   %put %unquote(&wab_error) The &temp_lib..etl_detail_dates data set %QCMPRES(
        with) dates from etl cannot be opened.;
   %goto ERREXIT;
%end;

%if %get_observation_count(indsn=&temp_lib..etl_detail_dates)=0  %then %do;
   %put %unquote(&wab_error) The &temp_lib..etl_detail_dates data set %QCMPRES(
        with) dates from etl contains 0 observations.;
   %goto ERREXIT;
%end;

%******************************************************************************;
%* Ensure that the data set containing dates processed in the ETL detail       ;
%* data set and the Web Analytics warehouse exists, and can be opened.         ;
%******************************************************************************;

%if %sysfunc(exist(&temp_lib..update_dates))=0 %then %do;
   %put %unquote(&wab_error) The &temp_lib..update_dates data set %QCMPRES(
                 with) dates for which summaries will be %QCMPRES(
                 updated) does not exist.;
   %goto ERREXIT;
%end;

%locktest(dset=&temp_lib..update_dates);
%if &lock_sw ne 0 %then %do;
   %put %unquote(&wab_error) The &temp_lib..update_dates data set %QCMPRES(
                 with) dates for which summaries will be %QCMPRES(
                 updated) cannot be opened.;
   %goto ERREXIT;
%end;

%******************************************************************************;
%* Ensure that the data set containing dates processed in the ETL detail       ;
%* data set but not in the Web Analytics warehouse exists, and can be opened.  ;
%******************************************************************************;

%if %sysfunc(exist(&temp_lib..new_dates))=0 %then %do;
   %put %unquote(&wab_error) The &temp_lib..new_dates data set %QCMPRES(
                 with) dates for which summaries will be %QCMPRES(
                 added) does not exist.;
   %goto ERREXIT;
%end;

%locktest(dset=&temp_lib..new_dates);
%if &lock_sw ne 0 %then %do;
   %put %unquote(&wab_error) The &temp_lib..new_dates data set %QCMPRES(
                 with) dates for which summaries will be %QCMPRES(
                 added) cannot be opened.;
   %goto ERREXIT;
%end;

%******************************************************************************;
%* Write a warning to the log if the data set containing invalid dates can not ;
%* be opened. These dates exist in etl detail, Web Analytics session data, but ;
%* not in the Web Analytics detail data.                                       ;
%******************************************************************************;

%if %sysfunc(exist(&temp_lib..invalid_dates))=0 %then %do;
   %put %unquote(&wab_warning) The &temp_lib..invalid_dates data set %QCMPRES(
                 with) invalid dates does not exist.;
%end;

%locktest(dset=&temp_lib..invalid_dates);
%if &lock_sw ne 0 %then %do;
   %put %unquote(&wab_warning) The &temp_lib..invalid_dates data set %QCMPRES(
                 with) invalid dates cannot be opened.;
%end;

%******************************************************************************;
%* Get the number of dates which need to be updated, created and are invalid   ;
%* and store them into sas macro variables.                                    ;
%******************************************************************************;

data _null_;
   num_etl_dates=%get_observation_count(indsn=&temp_lib..etl_detail_dates);
   num_update_dates=%get_observation_count(indsn=&temp_lib..update_dates);
   num_new_dates=%get_observation_count(indsn=&temp_lib..new_dates);
   num_invalid_dates=%get_observation_count(indsn=&temp_lib..invalid_dates);
   call symput("_wab_num_etl_dates_",num_etl_dates);
   call symput("_wab_num_update_dates_",num_update_dates);
   call symput("_wab_num_new_dates_",num_new_dates);
   call symput("_wab_num_invalid_dates_",num_invalid_dates);
run;

%******************************************************************************;
%* Store each date in etl detail into a separate macro variable.               ;
%******************************************************************************;

%if &_wab_num_etl_dates_ gt 0 %then %do;

   data _null_;

      length etl_date_list $ 4096;
      format etl_date_list $4096.;
      informat etl_date_list $4096.;
      retain etl_date_list " ";

      set &temp_lib..etl_detail_dates end=eof;
      tempdate=strip(put(date,6.));

      call symput("_wab_etl_sas_date_"||trim(left(put(_n_,6.)))
                 ,tempdate);

      call symput("_wab_etl_date_"||trim(left(put(_n_,6.)))
                 ,trim(left(put(date,yymmddn8.))));

      if not eof then do;

         etl_date_list=strip(compress(tempdate
                                     ||","
                                     ||etl_date_list));
      end;

      else do;

         etl_date_list=strip(compress(etl_date_list
                                        ||tempdate));

         call symput("_wab_etl_sas_date_list_"
                    ,strip(etl_date_list));

      end;

   run;

%end;

%******************************************************************************;
%* Store each date whose detail is to be updated into a separate macro         ;
%* variable.                                                                   ;
%******************************************************************************;

%if &_wab_num_update_dates_ gt 0 %then %do;

   data _null_;

      length update_date_list $ 4096;
      format update_date_list $4096.;
      informat update_date_list $4096.;
      retain update_date_list " ";


      set &temp_lib..update_dates end=eof;
      tempdate=strip(put(date,6.));
      call symput("_wab_update_sas_date_"||trim(left(put(_n_,6.)))
                 ,tempdate);

      call symput("_wab_update_date_"||trim(left(put(_n_,6.)))
                 ,trim(left(put(date,yymmddn8.))));
      if not eof then do;

         update_date_list=strip(compress(tempdate
                                        ||","
                                        ||update_date_list));
      end;

      else do;

         update_date_list=strip(compress(update_date_list
                                        ||tempdate));

         call symput("_wab_update_sas_date_list_"
                    ,strip(update_date_list));


      end;

   run;

%end;

%******************************************************************************;
%* Store each date whose detail is to be created into a separate macro         ;
%* variable.                                                                   ;
%******************************************************************************;

%if &_wab_num_new_dates_ gt 0 %then %do;

   data _null_;

      length new_date_list $ 4096;
      format new_date_list $4096.;
      informat new_date_list $4096.;
      retain new_date_list " ";

      set &temp_lib..new_dates end=eof;
      tempdate=strip(put(date,6.));

      call symput("_wab_new_sas_date_"||trim(left(put(_n_,6.)))
                 ,tempdate);
      call symput("_wab_new_date_"||trim(left(put(_n_,6.)))
                 ,trim(left(put(date,yymmddn8.))));

      if not eof then do;

         new_date_list=strip(compress(tempdate
                                     ||","
                                     ||new_date_list));
      end;

      else do;

         new_date_list=strip(compress(new_date_list
                                        ||tempdate));

         call symput("_wab_new_sas_date_list_"
                    ,strip(new_date_list));

      end;

   run;

%end;


%******************************************************************************;
%* Create macro variables for each date that is to be updated or created.      ;
%******************************************************************************;

data &temp_lib..all_dates;

   set &temp_lib..update_dates
       &temp_lib..new_dates end=eof;

   call symput("_wab_date_"||trim(left(put(_n_,6.)))
              ,trim(left(put(date,yymmddn8.))));

   if eof then call symput("_wab_num_dates_"
                          ,trim(left(put(_n_,6.))));
run;

proc means data=&temp_lib..all_dates noprint min max;
   var date;
   output out=&temp_lib..wab_min_max_process_dates
          min=wab_min_date
          max=wab_max_date;
run;

data _null_;
   set &temp_lib..wab_min_max_process_dates;
   call symput("_wab_min_process_date_",wab_min_date);
   call symput("_wab_max_process_date_",wab_max_date);
run;


%******************************************************************************;
%* Store each date in etl detail into a separate macro variable.               ;
%******************************************************************************;


%if &_wab_num_invalid_dates_ gt 0 %then %do;

   data _null_;
      set &temp_lib..invalid_dates end=eof;
      call symput("_wab_invalid_date_"||trim(left(put(_n_,6.)))
                 ,trim(left(put(date,yymmddn8.))));
   run;


   %do wab_iter=1 %to &_wab_num_invalid_dates_;
   %***************************************************************************;
   %* Write notification message to the log.                                   ;
   %***************************************************************************;
      %put %unquote(&wab_warning) %QCMPRES(
           &temp_lib..detail_&&_wab_invalid_date_&wab_iter) %QCMPRES(
           is) invalid because there is no %QCMPRES(
           corresponding) DETAIL data set for the date %QCMPRES(
           &&_wab_invalid_date_&wab_iter) but there is a corresponding %QCMPRES(
           SESSION) data set for the date &&_wab_invalid_date_&wab_iter.;

   %end;

%end;

%******************************************************************************;
%* If there are no dates that need to be updated or created in the web mart    ;
%* from ETL detail then exit processing.                                       ;
%******************************************************************************;

%if &_wab_num_update_dates_ le 0 and
   &_wab_num_new_dates_ le 0 %then %do;
   %put %unquote(&wab_error) There are no dates in ETL detail that %QCMPRES(
   need) to be created or updated in the Web Analytics web mart.;
   %goto ERREXIT;
%end;


%******************************************************************************;
%* SECTION: II - Create Analysis Data Sets                                     ;
%******************************************************************************;

%******************************************************************************;
%* MODULE: II_A - Partition ETL Detail                                         ;
%* SUPPORT:frroed                                                              ;
%* Partition ETL Detail for appending and creating date partitioned Web        ;
%* Analytics Detail.                                                           ;
%******************************************************************************;
%let module=II_A;
%let macname=%scan(&macname,1,_)_&module;


%******************************************************************************;
%* Create one Partitioned ETL data set per date in ETL detail.                 ;
%******************************************************************************;
data %do wab_iter=1 %to &_wab_num_etl_dates_;
        &temp_lib..detail_&&_wab_etl_date_&wab_iter
     %end;;
   set &detail;
   %***************************************************************************;
   %* convert all missing STATUS_CODEs to zero                                 ;
   %***************************************************************************;
   if status_code le .z then
      status_code=0;
   select (date);
      %do j=1 %to &_wab_num_etl_dates_;
         when (&&_wab_etl_sas_date_&j)
               output &temp_lib..detail_&&_wab_etl_date_&j;
      %end;
      otherwise;
   end;
   %***************************************************************************;
   %* make sure that STATUS_CODE does not have a custom format.  the data      ;
   %*    set that WAETL inherits from WebHound has STATUS_CODE assigned the    ;
   %*    STATUS format, which causes WAETL to fail if one of the STATUS_CODEs  ;
   %*    it encounters is not defined in the format.  because SWA cannot       ;
   %*    update the STATUS format, WAETL removes the format for STATUS_CODE.   ;
   %***************************************************************************;
   format status_code 3.;
run;

%******************************************************************************;
%* Process each Partitioned ETL data set.                                      ;
%******************************************************************************;
%do wab_iter=1 %to &_wab_num_etl_dates_;

   %***************************************************************************;
   %* Check if the Partitioned ETL is sorted correctly.                        ;
   %***************************************************************************;
   data _null_;
      %************************************************************************;
      %* Keep only the Sort Variables to reduce I/O.                           ;
      %************************************************************************;
      set &temp_lib..detail_&&_wab_etl_date_&wab_iter
         (keep=session_id
               datetime);
      by session_id
         datetime;
   run;

   %***************************************************************************;
   %* Sort the Partitioned ETL (if necessary)                                  ;
   %*    Note: the previous DATA _NULL_ step sets &SYSERR to 1012 if the data  ;
   %*          set referenced in its SET statement is not in the order         ;
   %*          specified in the BY statement.                                  ;
   %***************************************************************************;
   %if &syserr eq 1012 %then %do;

      proc sort data=&temp_lib..detail_&&_wab_etl_date_&wab_iter;
         by session_id
            datetime;
      run;

   %end;

   %***************************************************************************;
   %* Create the STATCODE format that provides an index for each STATUS_CODE   ;
   %*    in the date-partitioned detail data.                                  ;
   %* Create the $REQFILE format that provides an index for each               ;
   %*    REQUESTED_FILE in the date-partitioned detail data.                   ;
   %***************************************************************************;
   proc summary data=&temp_lib..detail_&&_wab_etl_date_&wab_iter
         chartype
         missing
         n;
      class status_code
            requested_file;
      output out=&temp_lib..all_status_codes
                 (where=(_type_='10') keep=status_code _freq_ _type_)
                ;
      output out=&temp_lib..all_requested_files
                 (where=(_type_='01') keep=requested_file _freq_ _type_)
                ;
   run;
   data &temp_lib..all_status_codes_fmt;
      set &temp_lib..all_status_codes;

      length label $ 7;

      fmtname="statcode";
      label  =strip(put(_n_,7.));

      rename status_code=start;

      keep fmtname
           status_code
           label;

   run;
   proc format cntlin=&temp_lib..all_status_codes_fmt;
   run;
   data &temp_lib..all_requested_files_fmt;
      set &temp_lib..all_requested_files;

      length label $ 7;

      fmtname="$reqfile";
      label  =strip(put(_n_,7.));

      rename requested_file=start;

      keep fmtname
           requested_file
           label;

   run;
   proc format cntlin=&temp_lib..all_requested_files_fmt;
   run;


   %***************************************************************************;
   %* Assign macro variables for ARRAYs in the next DATA Step:                 ;
   %*    REQ_FILE_CNT: the number of REQUESTED_FILEs in the source data set    ;
   %*    STAT_CDE_CNT: the number of STATUS_CODEs in the source data set       ;
   %*    FLAG_CNT:     the number of REQUESTED_FILEs times the number of       ;
   %*                  STATUS_CODEs (the total number of elements in an array  ;
   %*                  of flags that track STATUS_CODE by REQUESTED_FILE       ;
   %*                  within SESSION_ID)                                      ;
   %***************************************************************************;
   %let req_file_cnt=%get_observation_count
      (indsn=&temp_lib..all_requested_files_fmt);
   %let stat_cde_cnt=%get_observation_count
      (indsn=&temp_lib..all_status_codes_fmt);
   %let flag_cnt=%eval(&req_file_cnt*&stat_cde_cnt);


   %***************************************************************************;
   %* Make sure that the Partitioned ETL has the additional variables it needs ;
   %*    for the standard analyses.                                            ;
   %***************************************************************************;
   data &temp_lib..detail_&&_wab_etl_date_&wab_iter;

      format Initial_Reference_by_Session
             Bytes_Sent
             File_Count
             Page_Count
             Entry_Point
             Redirect_Count
             Client_Error_Count
             Server_Error_Count
             Entry_Page_Count
             Exit_Page_Count
             Err_000_Count
             Err_302_Count
             Err_304_Count
             Err_400_Count
             Err_401_Count
             Err_403_Count
             Err_404_Count
             Err_405_Count
             Err_408_Count
             Err_500_Count
             Err_501_Count nlnum15.
             Start_of_Week
             Start_of_Month
             Start_of_Quarter
             Start_of_Year date9.
             Dow_ID $2.;

      set &temp_lib..detail_&&_wab_etl_date_&wab_iter;
         by session_id
            hour;

      retain Sequence 0
             statcde1-statcde&stat_cde_cnt
             flag1-flag&flag_cnt
            ;

      %* define an array of STATUS_CODE indicator variables                    ;
      array statcdes {1:&stat_cde_cnt} statcde1-statcde&stat_cde_cnt;
      %* define an array of REQUESTED_FILE by STATUS_CODE indicator variables  ;
      array reqf_stcd{1:&req_file_cnt,1:&stat_cde_cnt} flag1-flag&flag_cnt;

      Entry_Page_Count=0;

      if first.session_id then do;
         Sequence=0;
         Entry_Page_Count=1;
         do reqf=1 to &req_file_cnt;
            do stcd=1 to &stat_cde_cnt;
               reqf_stcd{reqf,stcd}=0;
            end;
         end;
      end;

      %************************************************************************;
      %* to count each STATUS_CODE only once per hour for STATUS_CODE_BY_HOUR, ;
      %*    initialize all indicator variables to 0 -- not encountered -- at   ;
      %*    the head of each hour                                              ;
      %************************************************************************;
      if first.hour then do;
         do i=1 to &stat_cde_cnt;
            statcdes{i}=0;
         end;
      end;

      Sequence=Sequence+1;
      Exit_Page_Count=0;
      Redirect_Count=(300<=status_code<=399);
      Client_Error_Count=(400<=status_code<=499);
      Server_Error_Count=(500<=status_code);

      Err_000_Count=(status_code=0);
      Err_302_Count=(status_code=302);
      Err_304_Count=(status_code=304);
      Err_400_Count=(status_code=400);
      Err_401_Count=(status_code=401);
      Err_403_Count=(status_code=403);
      Err_404_Count=(status_code=404);
      Err_405_Count=(status_code=405);
      Err_408_Count=(status_code=408);
      Err_500_Count=(status_code=500);
      Err_501_Count=(status_code=501);

      %************************************************************************;
      %* identify the current STATUS_CODE/REQUESTED_FILEs location in the      ;
      %*    array of indicator variables and set REQFILE_STATCDE to 1 if this  ;
      %*    is its first occurance during the current SESSION_ID               ;
      %************************************************************************;
      reqf=input(put(requested_file,$reqfile.),7.);
      stcd=input(put(status_code,statcode.),7.);
      if reqf_stcd{reqf,stcd} eq 0 then do;
         reqfile_statcde=1;
         reqf_stcd{reqf,stcd}=1;
      end;
      else
         reqfile_statcde=0;

      %************************************************************************;
      %* Normalize URI/URL variables.                                          ;
      %************************************************************************;
      requested_file=left(trim(compress(lowcase(requested_file))));
      referrer=left(trim(compress(lowcase(referrer))));
      referrer_domain=left(trim(compress(lowcase(referrer_domain))));
      browser=left(trim(lowcase(browser)));
      browser_version=left(trim(lowcase(browser_version)));
      if browser_version eq ' ' then
         browser_version='NA';
      platform=left(trim(lowcase(platform)));

      %************************************************************************;
      %* Create date variables for summaries.                                  ;
      %************************************************************************;
      start_of_week   =intnx("week&wab_week_shift",date,0);
      start_of_month  =intnx('month',date,0);
      start_of_quarter=intnx('qtr',date,0);
      start_of_year   =intnx('year',date,0);

      if compbl(Referrer)=" " then Referrer="No Referrer";

      if compbl(Referrer_Domain)=" "
         then Referrer_Domain="No Referrer Domain";

      weekday=weekday(date);
      dow_id=strip(put(weekday,2.));

      %************************************************************************;
      %* identify the current STATUS_CODEs location in the array of indicator  ;
      %*    variables and set STATUS_CODE_BY_HOUR to 1 if this is its first    ;
      %*    occurance during the current hour                                  ;
      %************************************************************************;
      stat_idx=input(put(status_code,statcode.),3.);
      if statcdes{stat_idx} eq 0 then do;
         statcdes{stat_idx}=1;
         status_code_by_hour=1;
      end;
      else
         status_code_by_hour=0;

      if last.session_id then do;
         Exit_Page_Count=1;
      end;

      label Page_Count="Page Count"
            Redirect_Count="Redirect Count"
            Client_Error_Count="Client Error Count"
            Server_Error_Count="Server Error Count"
            Entry_Page_Count="Entry Page Count"
            Exit_Page_Count="Exit Page Count"
            Err_000_Count="0 Count: Missing Status Code"
            Err_302_Count="302 Count: Found"
            Err_304_Count="304 Count: Not Modified"
            Err_400_Count="400 Count: Bad Request"
            Err_401_Count="401 Count: Unauthorized"
            Err_403_Count="403 Count: Forbidden"
            Err_404_Count="404 Count: Not Found"
            Err_405_Count="405 Count: Method Not Allowed"
            Err_408_Count="408 Count: Request Time-out"
            Err_500_Count="500 Count: Internal Server Error"
            Err_501_Count="501 Count: Not Implemented"
            Initial_Reference_by_Session="Session Count"
            Start_of_Week="Start of Week"
            Start_of_Month="Start of Month"
            Start_of_Quarter="Start of Quarter"
            Start_of_Year="Start of Year"
            Dow_ID="Day of Week ID"
            status_code_by_hour='Indicator for hourly STATUS_CODE use'
            reqfile_statcde='Indicator for STATUS_CODE by REQUESTED_FILE use within SESSION_ID'
           ;

      drop weekday
           statcde1-statcde&stat_cde_cnt
           i
           reqf
           stcd
           flag1-flag&flag_cnt
          ;

   run;


   %***************************************************************************;
   %* there is a need to establish indicator variables that flag events one    ;
   %*    time per SESSION_ID.  these indicator variables are processed by the  ;
   %*    Summary Engine to count how many sessions have a certain feature.     ;
   %*       EXAMPLE: to count how many sessions have status codes of 404       ;
   %*                (page not found), we establish an INIT_BY_SESS_x variable ;
   %*                that is set to 1 the first time a STATUS_CODE of 404 is   ;
   %*                encountered in a session, and has a value of 0 all other  ;
   %*                times.  By running PROC SUMMARY with INIT_BY_SESS_x as    ;
   %*                the analysis var and STATUS_CODE as the class var, we can ;
   %*                then count how many sessions have STATUS_CODE of 404.     ;
   %***************************************************************************;

   %***************************************************************************;
   %* create a format for each variable in WAB_INIT_BY_SESS_VARS to indicate   ;
   %*    whether the variable is numeric or character.  this is necessary to   ;
   %*    create the right kind of format for each variable.                    ;
   %***************************************************************************;
   %let wab_ibsv_blank_delimit=%upcase
                              (%sysfunc
                              (translate
                              (%quote(&wab_init_by_sess_vars),' ',',')
                             ));
   proc sql;
      create table &temp_lib..wab_ibsv_var_type as
         select upcase(name) as start,
                type as label,
                '$vartype' as fmtname
         from dictionary.columns
            where libname eq upcase("&temp_lib") and
                  memname eq upcase("detail_&&_wab_etl_date_&wab_iter") and
                  index("&wab_ibsv_blank_delimit",upcase(strip(name)));
   quit;
   proc format cntlin=&temp_lib..wab_ibsv_var_type;
   run;


   %***************************************************************************;
   %* create macro vars for each variable in WAB_INIT_BY_SESS_VARS:            ;
   %*     a) v_name - the SAS var name                                         ;
   %*     b) v_prfx - the format prefix ($ or nullstring)                      ;
   %*     c) v_sffx - the format suffix (a letter: a to z)                     ;
   %*     d) v_fnam - the name of the format that creates an id value for a    ;
   %*                 corresponding value of V_NAME                            ;
   %*     e) v_fmt  - v_fnam with the terminal period appended                 ;
   %*     f) v_type - the _type_ mask for PROC SUMMARY                         ;
   %* each variable in WAB_INIT_BY_SESS_VARS has its own numeric suffix that   ;
   %* identifies the macro var as belonging to it.                             ;
   %* also, create the IBSV_CNT macro var with the # of WAB_INIT_BY_SESS_VARS  ;
   %***************************************************************************;
   data _null_;
      length v_type
             wab_init_by_sess_vars $ 1024
             v_sffx
             v_prfx                $    1
             v_fnam                $    8
             v_fmt                 $    9
             v_name                $   32;

      wab_init_by_sess_vars="&wab_init_by_sess_vars";
      i=0;
      do until (wab_init_by_sess_vars eq ' ');
         i+1;
         v_name=scan(wab_init_by_sess_vars,1,',');
         v_sffx=byte(96+i);
         if put(upcase(v_name),$vartype.) eq 'char' then
            v_prfx='$';
         else
            v_prfx=' ';
         v_fnam=strip(strip(v_prfx)
                   || 'v_fmt'
                   || strip(v_sffx)
                     );
         v_fmt =strip(v_fnam)
             || '.';
         call symput('v_name'||strip(put(i,3.))
                    ,strip(v_name)
                    );
         call symput('v_prfx'||strip(put(i,3.))
                    ,strip(v_prfx)
                    );
         call symput('v_sffx'||strip(put(i,3.))
                    ,strip(v_sffx)
                    );
         call symput('v_fnam'||strip(put(i,3.))
                    ,strip(v_fnam)
                    );
         call symput('v_fmt'||strip(put(i,3.))
                    ,strip(v_fmt)
                    );
         wab_init_by_sess_vars=substr(wab_init_by_sess_vars,(2+length(v_name)));
      end;
      call symput('ibsv_cnt',strip(put(i,3.)));
      v_type=repeat('0',(i-1));
      do j=1 to i;
         substr(v_type,j,1)='1';
         call symput('v_type'||strip(put(j,3.))
                    ,strip(v_type)
                    );
         substr(v_type,j,1)='0';
      end;

   run;


   %***************************************************************************;
   %* execute a single PROC SUMMARY step to create an OUTPUT data set for each ;
   %*    variable in the WAB_INIT_BY_SESS_VARS list.                           ;
   %***************************************************************************;
   proc summary data=&temp_lib..detail_&&_wab_etl_date_&wab_iter
        sum missing chartype;

      class %sysfunc(translate(%quote(&wab_init_by_sess_vars),' ',','));

      types %sysfunc(translate(%quote(&wab_init_by_sess_vars),' ',','));

      var
         ;

      %do i=1 %to &ibsv_cnt;
         output out=&&v_name&i(
                where=(_type_="&&v_type&i")
                keep= &&v_name&i _type_)
                ;
      %end;

   run;

   %***************************************************************************;
   %* for each variable in the WAB_INIT_BY_SESS_VARS list, create a format to  ;
   %*    assign an ID for each of its values.  this id will be used to set a   ;
   %*    binary indicator in an array in a subsequent DATA step.               ;
   %***************************************************************************;
   %do i=1 %to &ibsv_cnt;
      data cntlin;
         set &&v_name&i
             end=eof;

         fmtname="&&v_fnam&i";
         start  =&&v_name&i;
         label  =strip(put(_n_,7.));

         if eof then
            call symput("v_cnt&i",strip(put(_n_,7.)));

         keep fmtname
              start
              label;

      run;
      proc format cntlin=cntlin;
      run;
   %end;

   %***************************************************************************;
   %* These macro variables make it convenient to use ranges to identify       ;
   %*    variable lists.                                                       ;
   %***************************************************************************;
   data _null_;
      do i=1 to &ibsv_cnt;
         call symput('xx'||strip(put(i,3.))
                    ,'1'
                    );
      end;
   run;

   %***************************************************************************;
   %* Create the initialize-by-session variables.                              ;
   %*    NOTE: this requires a second pass of the data set because the earlier ;
   %*          pass has the responsibility of normalizing several character    ;
   %*          variables, any of which could be the source for initialize-by-  ;
   %*          session indicators.                                             ;
   %***************************************************************************;
   data &temp_lib..detail_&&_wab_etl_date_&wab_iter;

      %* make sure that each INIT_BY_SESS_VAR has the standard format          ;
      format
      %do i=1 %to &ibsv_cnt;
         init_by_sess_&&v_sffx&i
      %end;
            nlnum15.;

      %* define one array for each INIT_BY_SESS_VAR                            ;
      %do i=1 %to &ibsv_cnt;
         array var_&&v_sffx&i {1:&&v_cnt&i}
               var_&&v_sffx&i&&xx&i-var_&&v_sffx&i&&v_cnt&i;
      %end;

      set &temp_lib..detail_&&_wab_etl_date_&wab_iter;
         by session_id;

      retain %do i=1 %to &ibsv_cnt;
                var_&&v_sffx&i&&xx&i-var_&&v_sffx&i&&v_cnt&i
             %end;
            ;

      if first.session_id then do;
         %* initialize each INIT_BY_SESS_VAR array                             ;
         %do i=1 %to &ibsv_cnt;
            do i=1 to &&v_cnt&i;
               var_&&v_sffx&i{i}=0;
            end;
         %end;
      end;

      %* for each INIT_BY_SESS_VAR...                                          ;
      %do i=1 %to &ibsv_cnt;
         %* locate the position of the value in the variables array            ;
         idx_&&v_sffx&i=input(put(&&v_name&i,&&v_fmt&i..),7.);
         %* if it has not been encountered in the sess., turn on indicator     ;
         if not var_&&v_sffx&i{idx_&&v_sffx&i} then do;
            var_&&v_sffx&i{idx_&&v_sffx&i}=1;
            init_by_sess_&&v_sffx&i=1;
         end;
         %* if it has already been encountered, turn off indicator             ;
         else
            init_by_sess_&&v_sffx&i=0;
      %end;

      drop i
           %do i=1 %to &ibsv_cnt;
              var_&&v_sffx&i&&xx&i-var_&&v_sffx&i&&v_cnt&i
              idx_&&v_sffx&i
           %end;
          ;

      label %do i=1 %to &ibsv_cnt;
               init_by_sess_&&v_sffx&i="INIT_BY_SESSION for &&v_name&i"
            %end;
           ;

   run;

   %***************************************************************************;
   %* Separate out any Tally/Spider records into a separate _TALLY data set.   ;
   %***************************************************************************;
   data &temp_lib..detail_&&_wab_etl_date_&wab_iter
        &temp_lib..detail_tally_&&_wab_etl_date_&wab_iter;
      set &temp_lib..detail_&&_wab_etl_date_&wab_iter;
      if (file_count gt 1                       and
          upcase(requested_file) eq '(NONPAGE)' and
          upcase(referrer) eq '(TALLY)'
         ) or
         (upcase(requested_file) eq '(SPIDER)' and
          upcase(referrer) eq '(TALLY)'
         ) then
         output &temp_lib..detail_tally_&&_wab_etl_date_&wab_iter;
      else
         output &temp_lib..detail_&&_wab_etl_date_&wab_iter;
   run;

%end;


%******************************************************************************;
%* MODULE: II_B - Create Web Analytics Session Data                            ;
%* SUPPORT:saskxs                                                              ;
%* Create the session data sets from previously appended and/or created        ;
%* Web Analytics detail                                                        ;
%******************************************************************************;
%let module=II_B;
%let macname=%scan(&macname,1,_)_&module;



%******************************************************************************;
%* For each date that whose Web Analytics kept detail has been updated also    ;
%* update sessions.                                                            ;
%******************************************************************************;

%do wab_iter=1 %to &_wab_num_dates_;

%******************************************************************************;
%* create SESS_VARS to inventory the SESSION_ID-related special variables and  ;
%*    the FSVn (First Session Variable), LSVn (Last Session Variable), and     ;
%*    SSVn (Summary Session Variable) macro variables.                         ;
%******************************************************************************;
data sess_vars;
   length varname
          name
          var_cat $ 32
          varlist $ 4096;

   varlist="&wab_first_sess_vars";
   varseq=0;
   do while (varlist gt ' ');
      varseq+1;
      varname=scan(varlist,1,',');
      call symput('fsv'||strip(put(varseq,2.))
                 ,strip(varname));
      varlist=substr(varlist,(2+length(varname)));
      name=upcase(varname);
      var_cat='first_sess_vars';
      output;
   end;
   call symput('fsv_cnt'
              ,strip(put(varseq,2.)));

   varlist="&wab_last_sess_vars";
   varseq=0;
   do while (varlist gt ' ');
      varseq+1;
      varname=scan(varlist,1,',');
      call symput('lsv'||strip(put(varseq,2.))
                 ,strip(varname));
      varlist=substr(varlist,(2+length(varname)));
      name=upcase(varname);
      var_cat='last_sess_vars';
      output;
   end;
   call symput('lsv_cnt'
              ,strip(put(varseq,2.)));

   varlist="&wab_sum_sess_vars";
   varseq=0;
   do while (varlist gt ' ');
      varseq+1;
      varname=scan(varlist,1,',');
      call symput('ssv'||strip(put(varseq,2.))
                 ,strip(varname));
      varlist=substr(varlist,(2+length(varname)));
      name=upcase(varname);
      var_cat='sum_sess_vars';
      output;
   end;
   call symput('ssv_cnt'
              ,strip(put(varseq,2.)));

   keep varname
        name
        var_cat;

run;


%******************************************************************************;
%* make sure that there is not any duplication across the 3 types of SESSION   ;
%*    variables                                                                ;
%******************************************************************************;
proc sort data=sess_vars;
   by varname;
run;
data uniq
     mult;
   set sess_vars;
   by varname;

   file log;
   if first.varname then
      if last.varname then
         output uniq;
      else do;
         output mult;
         put 'SESSION variable: '
             varname
         +10 'Variable Category: '
             var_cat;
      end;
   else do;
      output mult;
      put 'SESSION variable: '
          varname
      +10 'Variable Category: '
          var_cat;
   end;

run;
%if %get_observation_count(indsn=mult) ne 0 %then %do;
   %put %unquote(&wab_error) There are duplicate SESS_VARS.;
   %goto ERREXIT;
%end;


%******************************************************************************;
%* make sure that all SESSION variables are in the source data set             ;
%*   (&TEMP_LIB..DETAIL&&_WAB_DATE_&WAB_ITER)                                  ;
%******************************************************************************;
proc contents data=&temp_lib..detail_&&_wab_date_&wab_iter noprint
              out =varlist (keep=type
                                 name);
run;
data varlist;
   set varlist;
   name=upcase(name);
run;
proc sort data=varlist;
   by name;
run;
data both
     only_a
     only_b;
   merge varlist (in=in_a)
         uniq    (in=in_b);
   by name;

   if in_a then
      if in_b then
         output both;
      else
         output only_a;
   else do;
      file log;
      put 'SESSION variable: '
          varname
      +10 'Variable Category: '
          var_cat;
      output only_b;
   end;

run;
%if %get_observation_count(indsn=only_b) ne 0 %then %do;
   %put %unquote(&wab_error) There are SESS_VARS that are not in &temp_lib..detail_&&_wab_date_&wab_iter.;
   %goto ERREXIT;
%end;

%******************************************************************************;
%* make sure that all SUM_SESS_VARS are numeric                                ;
%******************************************************************************;
data non_numeric;
   set both;
   if index(upcase(var_cat),'SUM_') and type ne 1;

   file log;
   put 'SESSION variable: '
       varname
   +10 'Variable Category: '
       var_cat;

run;
%if %get_observation_count(indsn=non_numeric) ne 0 %then %do;
   %put %unquote(&wab_error) There are SUM_SESS_VARS that are not numeric.;
   %goto ERREXIT;
%end;

data &temp_lib..session_&&_wab_date_&wab_iter
         (keep=Client_Id
               Session_Id
               Visitor_Id
               First_User_Agent
               First_Browser
               First_Browser_Version
               First_Platform
               First_Requested_File
               First_Referrer
               First_Referrer_Domain
               First_Page_View_Ind
               Beg_Datetime
               Date Start_of_Week
               Start_of_Month
               Start_of_Quarter
               Dow_ID
               Start_of_Year
               Err_000_Count
               Err_302_Count
               Err_304_Count
               Err_400_Count
               Err_401_Count
               Err_403_Count
               Err_404_Count
               Err_405_Count
               Err_408_Count
               Err_500_Count
               Err_501_Count
               End_Datetime
               Last_Requested_File
               Last_Page_View_Ind
               File_Count Page_Count
               One_Hit_Session_Count
               Total_Bytes_Sent
               Duration Page_View_Le1_Ind
               Page_View_2to4_Ind
               Page_View_Ge5_Ind
               Session_Count
               Search_Term

               %do i=1 %to &ssv_cnt;
                  &&ssv&i
               %end;
               %do i=1 %to &fsv_cnt;
                  &&fsv&i
               %end;
               %do i=1 %to &lsv_cnt;
                  &&lsv&i
               %end;

                                 rename=(First_User_Agent=User_Agent
                                         First_Browser=Browser
                                         First_Browser_Version=Browser_Version
                                         First_Platform=Platform
                                         First_Referrer=Referrer
                                         First_Referrer_Domain=Referrer_Domain

                                        )
                                );

   length First_User_Agent $ 160
          First_Browser $ 40
          First_Browser_Version $ 8
          First_Platform $ 40
          First_Requested_File $ 1024
          Last_Requested_File $ 1024
          Referrer_Query_String_New  $ 1024
          Referrer_Query_String_Temp $ 1024
          Referrer_Query_String      $ 1024
          First_Referrer $ 1024
          First_Referrer_Domain $ 128
          Search_Term $ 1024
          Beg_Datetime
          First_Page_View_Ind
          Last_Page_View_Ind
          Page_Sequence
          File_Count
          Err_000_Count
          Err_302_Count
          Err_304_Count
          Err_400_Count
          Err_401_Count
          Err_403_Count
          Err_404_Count
          Err_405_Count
          Err_408_Count
          Err_500_Count
          Err_501_Count
          Session_Count 8;

   format First_Page_View_Ind
          Last_Page_View_Ind
          File_Count
          Page_Count
          One_Hit_Session_Count
          Total_Bytes_Sent
          Page_View_Le1_Ind
          Page_View_2to4_Ind
          Page_View_Ge5_Ind
          Err_000_Count
          Err_302_Count
          Err_304_Count
          Err_400_Count
          Err_401_Count
          Err_403_Count
          Err_404_Count
          Err_405_Count
          Err_408_Count
          Err_500_Count
          Err_501_Count
          Duration
          Session_Count nlnum15.
          Status_Code 3.;

   set &temp_lib..detail_&&_wab_date_&wab_iter
         (rename=(Err_000_Count=Err_000
                  Err_302_Count=Err_302
                  Err_304_Count=Err_304
                  Err_400_Count=Err_400
                  Err_401_Count=Err_401
                  Err_403_Count=Err_403
                  Err_404_Count=Err_404
                  Err_405_Count=Err_405
                  Err_408_Count=Err_408
                  Err_500_Count=Err_500
                  Err_501_Count=Err_501
                 )
         );

   array Parm_Names{100} $ 128 _TEMPORARY_;
   array Parm_Values{100} $ 128 _TEMPORARY_;

   retain First_User_Agent First_Browser  First_Browser_Version
          First_Platform First_Requested_File First_Referrer
          First_Referrer_Domain Search_Term  " "

          Beg_Datetime Page_Sequence File_Count Total_Bytes_Sent
          Err_000_Count
          Err_302_Count
          Err_304_Count
          Err_400_Count
          Err_401_Count
          Err_403_Count
          Err_404_Count
          Err_405_Count
          Err_408_Count
          Err_500_Count
          Err_501_Count
          First_Page_View_Ind 0
          %do i=1 %to &fsv_cnt;
            hold_fsv&i
          %end;
         ;

   by Session_Id;

   format Beg_Datetime End_Datetime datetime.;

   label Err_000_Count="0 Count: Missing Status Code"
         Err_302_Count="302 Count: Found"
         Err_304_Count="304 Count: Not Modified"
         Err_400_Count="400 Count: Bad Request"
         Err_401_Count="401 Count: Unauthorized"
         Err_403_Count="403 Count: Forbidden"
         Err_404_Count="404 Count: Not Found"
         Err_405_Count="405 Count: Method Not Allowed"
         Err_408_Count="408 Count: Request Time-out"
         Err_500_Count="500 Count: Internal Server Error"
         Err_501_Count="501 Count: Not Implemented"
         Total_Bytes_Sent="Total Bytes Sent"
         Page_Sequence="Page View Sequence"
         One_Hit_Session_Count="One Hit Session"
         File_Count="Hit Count"
         Page_Count="Page Count"
         Duration="Duration in Seconds"
         First_Platform="Platform"
         First_Referrer="Referrer"
         First_Referrer_Domain="Referrer Domain"
         First_User_Agent="User Agent"
         First_Requested_File="First Requested File"
         First_Page_View_Ind="First Page View Session"
         Last_Requested_File="Last Requested File"
         Last_Page_View_Ind="Last Page View Session"
         End_Datetime="Ending Session Datetime"
         Beg_Datetime="Beginning Session Datetime"
         First_Browser="Browser"
         First_Browser_Version="Browser Version"
         Page_View_Le1_Ind="0-1 Page Session Count"
         Page_View_2to4_Ind="2-4 Page Session Count"
         Page_View_Ge5_Ind="5 or Greater Page Session Count"
         Session_Count="Session Count";

   %***************************************************************************;
   %* Initialize Count and Indicator Variables. Initialize First Referrer,     ;
   %* First Referrer Domain, First Browser, First Requested File, etc.         ;
   %***************************************************************************;

   if first.Session_Id
      then
      do;

      %do i=1 %to &ssv_cnt;
         hold_ssv&i=0;
      %end;
      %do i=1 %to &fsv_cnt;
         hold_fsv&i=&&fsv&i;
      %end;

      Err_000_Count=0;
      Err_302_Count=0;
      Err_304_Count=0;
      Err_400_Count=0;
      Err_401_Count=0;
      Err_403_Count=0;
      Err_404_Count=0;
      Err_405_Count=0;
      Err_408_Count=0;
      Err_500_Count=0;
      Err_501_Count=0;
      Total_Bytes_Sent=0;
      Page_Sequence=0;

      Beg_Datetime = Datetime;
      First_Page_View_Ind=Page_Count;
      First_Requested_File = Requested_File;
      First_User_Agent = User_Agent;
      First_Browser = Browser;
      First_Browser_Version = Browser_Version;
      First_Platform = Platform;

      if Referrer=" " then Referrer="No Referrer";
      else First_Referrer = Referrer;

      if Referrer_Domain=" " then Referrer_Domain="No Referrer Domain";
      else First_Referrer_Domain = Referrer_Domain;

      Referrer_Query_String=scan(Referrer,2,"?");
      Search_Term=" ";
      num_nv_pairs=countc(Referrer_Query_String,'=');

      Referrer_Query_String_Temp=tranwrd(Referrer_Query_String
                                        ,'=&'
                                        ,'=%20&'
                                        );

      Referrer_Query_String_New=strip(prxchange('s/[&=]/ /'
                                               ,-1
                                               ,Referrer_Query_String_Temp));

      %************************************************************************;
      %* Parse Referrer Search Terms.                                          ;
      %************************************************************************;

      if num_nv_pairs > 0 then do;

         do i=1 to num_nv_pairs;

            Parm_Names{i}=urldecode(strip(scan(Referrer_Query_String_New
                                              ,(2*i)-1," ")));
            Parm_Values{i}=urldecode(strip(scan(Referrer_Query_String_New
                                               ,(2*i)
                                               ," ")));

            if prxmatch('/^(\d)+$/'
                       ,Parm_Values{i})

               then continue;

            if prxmatch('/^(.*txt.*|k[0-9]|search[0-9])$/'
                       ,Parm_Names{i})
               or

               lowcase(Parm_Names{i})     in ("_searchkey",
                                              "form_keyword",
                                              "ix",
                                              "k",
                                              "key",
                                              "key_word",
                                              "keyword",
                                              "keywords",
                                              "kw",
                                              "mt",
                                              "name",
                                              "p",
                                              "q",
                                              "qhqn",
                                              "qkw",
                                              "qry",
                                              "qry_str",
                                              "qt",
                                              "query",
                                              "queryterm",
                                              "querytext",
                                              "rawto",
                                              "s",
                                              "sc",
                                              "search",
                                              "search_string",
                                              "search_term",
                                              "searchfor",
                                              "searchstring",
                                              "searchwords",
                                              "sitesearchquery",
                                              "ss",
                                              "sstring",
                                              "str",
                                              "string",
                                              "strsrch",
                                              "term",
                                              "terms",
                                              "userquery"
                                              )
               then

                  Search_Term=strip(Search_Term
                              ||" "
                              ||prxchange('s/[;"]/ /'
                                         ,-1
                                         ,prxchange("s/[']/ /"
                                                   ,-1
                                                   ,Parm_Values{i})));


         end;

      end;

      if Search_Term=" " then Search_Term="No Search Term";

   end;

   %***************************************************************************;
   %* Increment the Error Count Variables based on a requests status code.     ;
   %***************************************************************************;

   select(Status_Code);

     when(  0) Err_000_Count=Err_000_Count + 1;
     when(302) Err_302_Count=Err_302_Count + 1;
     when(304) Err_304_Count=Err_304_Count + 1;
     when(400) Err_400_Count=Err_400_Count + 1;
     when(401) Err_401_Count=Err_401_Count + 1;
     when(403) Err_403_Count=Err_403_Count + 1;
     when(404) Err_404_Count=Err_404_Count + 1;
     when(405) Err_405_Count=Err_405_Count + 1;
     when(408) Err_408_Count=Err_408_Count + 1;
     when(500) Err_500_Count=Err_500_Count + 1;
     when(501) Err_501_Count=Err_501_Count + 1;

     otherwise;

   end;

   %***************************************************************************;
   %* Sum the number of bytes sent for each request into a total for each      ;
   %* session. Also increment the page count for each record when the request  ;
   %* is a valid page.                                                         ;
   %***************************************************************************;

   Total_Bytes_Sent=sum(Total_Bytes_Sent,Bytes_Sent);
   Page_Sequence = Page_Sequence + Page_Count;

   %do i=1 %to &ssv_cnt;
      hold_ssv&i+&&ssv&i;
   %end;


   %***************************************************************************;
   %* Create the retrieve last request information for each session.           ;
   %***************************************************************************;
   if last.Session_Id
      then
      do;

      %do i=1 %to &ssv_cnt;
         &&ssv&i=hold_ssv&i;
      %end;
      %do i=1 %to &fsv_cnt;
         &&fsv&i=hold_fsv&i;
      %end;

      End_Datetime = Datetime;
      Last_Requested_File = Requested_File;
      Last_Page_View_Ind = Page_Count;
      File_Count = Sequence;

      One_Hit_Session_Count=(File_Count=1);
      Page_View_Le1_Ind=(Page_Sequence <= 1);
      Page_View_2to4_Ind=(2<= Page_Sequence < 5);
      Page_View_Ge5_Ind=(Page_Sequence >= 5);

      Page_Count=Page_Sequence;
      Duration=End_Datetime-Beg_Datetime;

      Session_Count=1;

      output;

   end;

run;

%end;

%******************************************************************************;
%* MODULE: II_C - Create Web Analytics Summary Data                            ;
%* SUPPORT:saskxs                                                              ;
%* Create the summary data sets from Web Analytics detail and session data.    ;
%******************************************************************************;
%let module=II_C;
%let macname=%scan(&macname,1,_)_&module;

%******************************************************************************;
%* Convert the WAADMSUM metadata into the WASUMENG format.                     ;
%******************************************************************************;
%waadmsum(meta_in=&wab_admsum
          ,meta_out=&temp_lib..wasumeng
          );

%******************************************************************************;
%* Call the summary macro to produce proc summary files for dated input files. ;
%******************************************************************************;
%wasumeng(meta_dsn=&wab_metadsn
         ,partition_flag=1
         ,out_summary_file=&temp_store&separator.wab_summary
         ,retcode=wab_rc
         ,summcnt=wab_summary_count
         );

%******************************************************************************;
%* If there are failures then exit web analytics macro.                        ;
%******************************************************************************;

%if &wab_rc ne 0 %then
%goto ERREXIT;

%******************************************************************************;
%* Include each file if it exists. If it does not exist fail.                  ;
%******************************************************************************;
%do wab_iter=1 %to &wab_summary_count;

   %let summid=%sysfunc(putn(&wab_iter,z3.));

   %if %sysfunc(fileexist(&temp_store&separator.wab_summary&summid..sas))
      %then %do;
         %include "&temp_store&separator.wab_summary&summid..sas";

         %if &wab_rc ne 0 %then %do;
            %put %unquote(&wab_error) The PROC SUMMARY  -- %QCMPRES(
                  &temp_store&separator.wab_summary&summid..sas) %QCMPRES(--
                  has) been created but cannot be INCLUDEd.;
            %goto ERREXIT;
         %end;
   %end;

   %else %do;
      %put %unquote(&wab_error) The location for the PROC SUMMARY %QCMPRES(
      step) &temp_store&separator.wab_summary&summid..sas does not exist.;
      %goto ERREXIT;
   %end;

%end;

%******************************************************************************;
%* Get a list of unique output data sets. Store the full names, memnames and   ;
%* number of unique output data sets which were created from date partitioned  ;
%* input data sets.                                                            ;
%******************************************************************************;

proc sort data=&temp_lib.._wab_meta_data_(keep=output_table)
          out=&temp_lib.._wab_meta_data_nodup
          nodupkey;
   by output_table;
run;


%if &syserr gt 4 %then %do;
   %put %unquote(&wab_error) The metadata data set containing %QCMPRES(
         the) summary output table names could not be sorted.;
   %goto ERREXIT;
%end;

data _null_;

   retain library_error 0;

   set &temp_lib.._wab_meta_data_nodup end=eof;
   call symput(compress("_wab_summary_dsn_"||put(_n_,7.)),output_table);

   libname=lowcase(strip(scan(output_table,1,'.')));
   memname=lowcase(strip(scan(output_table,2,'.')));

   if scan(output_table,2,'.')="" then do;
      memname=scan(output_table,1,'.');
      libname="work";
   end;

   call symput(compress("_wab_summary_memname_"||put(_n_,7.))
              ,memname);

   if eof then
      call symput("_wab_num_summary_tables_",put(_n_,7.));

run;

%******************************************************************************;
%* For each summary which was generated from a date partitioned data set       ;
%* determine its time frame and append to the correct permanent data set in the;
%* warehouse.                                                                  ;
%******************************************************************************;
%do wab_iter=1 %to &_wab_num_summary_tables_;

    %if %sysfunc(exist(summary.&&_wab_summary_memname_&wab_iter)) %then %do;

       proc means data=summary.&&_wab_summary_memname_&wab_iter min max noprint;
          var date;
          output out=&temp_lib..min_current_date(keep=max_current_date
                                                      min_current_date)
          min=min_current_date
          max=max_current_date;
       run;

       %if &syserr gt 4 %then %do;
           %put %unquote(&wab_error) The PROC MEANS to determine the %QCMPRES(
                 available) data for %QCMPRES(
                 summary.&&_wab_summary_memname_&wab_iter) failed.;
           %goto ERREXIT;
       %end;


       data _null_;
          set &temp_lib..min_current_date;
          memname="&&_wab_summary_memname_&wab_iter";
          memname=strip(lowcase(memname));
          call symput("_wab_min_current_date_"
                     ,strip(put(min_current_date,5.))
                     );
          call symput("_wab_max_current_date_"
                     ,strip(put(max_current_date,5.))
                     );

          time_frame=strip(lowcase(reverse(scan(reverse(memname),1,"_"))));

          call symput("_wab_aging_where_","");

          if time_frame = "day" then do;
             if strip(scan(memname,1,"_"))="visitor" then
                frames_in_history=-1 * &wab_visitor_days_in_history;
             else
                frames_in_history=-1 * &wab_days_in_history;
             min_date=intnx(time_frame
                           ,max_current_date
                           ,frames_in_history);
             call symput("_wab_aging_where_"
                        ,"(where=(date ge ("||strip(put(min_date,5.))||")))");
          end;

          else if time_frame = "week" then do;
             frames_in_history=-1 * &wab_weeks_in_history;
             min_date=intnx(strip(time_frame) || "&wab_week_shift"
                           ,max_current_date
                           ,frames_in_history);
             call symput("_wab_aging_where_"
                        ,"(where=(date ge ("||strip(put(min_date,5.))||")))");
          end;

          else if time_frame = "month" then do;
             frames_in_history=-1 * &wab_months_in_history;
             min_date=intnx(time_frame
                           ,max_current_date
                           ,frames_in_history
                           ,'sameday');
             call symput("_wab_aging_where_"
                        ,"(where=(date ge ("||strip(put(min_date,5.))||")))");
          end;

          else if time_frame = "qtr" then do;
             frames_in_history=-1 * &wab_quarters_in_history;
             %* the INTERVAL argument for INTNX needs to be QUARTER            ;
             min_date=intnx('quarter'
                           ,max_current_date
                           ,frames_in_history
                           ,'sameday');
             call symput("_wab_aging_where_"
                        ,"(where=(date ge ("||strip(put(min_date,5.))||")))");
          end;

          else if time_frame = "year" then do;
             frames_in_history=-1 * &wab_years_in_history;
             min_date=intnx(time_frame
                           ,max_current_date
                           ,frames_in_history
                           ,'sameday');
             call symput("_wab_aging_where_"
                        ,"(where=(date ge ("||strip(put(min_date,5.))||")))");
          end;

       run;

       proc append base=&&_wab_summary_dsn_&wab_iter
                   data=summary.&&_wab_summary_memname_&wab_iter
                        &_wab_aging_where_ ;
       run;

       %if &syserr gt 4 %then %do;
           %put %unquote(&wab_error) The PROC APPEND for %QCMPRES(
                 &&_wab_summary_dsn_&wab_iter) and %QCMPRES(
                 summary.&&_wab_summary_memname_&wab_iter) failed.;
           %goto ERREXIT;
       %end;

    %end;

%end;


%******************************************************************************;
%* Call the summary macro to produce proc summary files for nondated input     ;
%* files.                                                                      ;
%******************************************************************************;

%wasumeng(meta_dsn=&wab_metadsn
         ,partition_flag=0
         ,out_summary_file=&temp_store&separator.wab_post_summary
         ,retcode=wab_rc
         ,summcnt=wab_summary_count
         );

%if &wab_rc ne 0 %then
%goto ERREXIT;

%******************************************************************************;
%* Include each file if it exists. If it does not exist fail.                  ;
%******************************************************************************;

%do wab_iter=1 %to &wab_summary_count;

   %let summid=%sysfunc(putn(&wab_iter,z3.));

   %if %sysfunc(fileexist(&temp_store&separator.wab_post_summary&summid..sas))
      %then %do;

         %include "&temp_store&separator.wab_post_summary&summid..sas";

         %if &wab_rc ne 0 %then %do;
            %put %unquote(&wab_error) The PROC SUMMARY  -- %QCMPRES(
                  &temp_store&separator.wab_post_summary&summid..sas) %QCMPRES(
                  -- has) been created but cannot be INCLUDEd.;
            %goto ERREXIT;
         %end;

   %end;

   %else %do;

      %put %unquote(&wab_error) The location for the PROC SUMMARY %QCMPRES(
      step) &temp_store&separator.wab_post_summary&summid..sas does not exist.;
      %goto ERREXIT;

   %end;

%end;


%******************************************************************************;
%* MODULE: II_D - Append and Copy Web Analytics detail and sessions data       ;
%* SUPPORT: frroed                                                             ;
%* Append SESSION/DETAIL data sets to their warehouse counterparts for dates   ;
%*    that already exist in the warehouse.  Copy SESSION/DETAIL data sets to   ;
%*    the warehouse for dates that are new.                                    ;
%******************************************************************************;
%let module=II_D;
%let macname=%scan(&macname,1,_)_&module;

%******************************************************************************;
%* Append to existing SESSION/DETAIL data sets for dates that are already in   ;
%*    the warehouse.  Because new variables can be introduced at any time,     ;
%*    the appending is done with a DATA step, not PROC APPEND.                 ;
%******************************************************************************;
%if &_wab_num_update_dates_ gt 0 %then %do;

   %do wab_iter=1 %to &_wab_num_update_dates_;
      %************************************************************************;
      %* Append new SESSION data sets                                          ;
      %************************************************************************;
      data dated.session_&&_wab_update_date_&wab_iter;
         set dated.session_&&_wab_update_date_&wab_iter
             &temp_lib..session_&&_wab_update_date_&wab_iter;
      run;
      %if &syserr gt 4 %then %do;
         %put %unquote(&wab_error) PROC APPEND for the updated %QCMPRES(
               session_&&_wab_update_date_&wab_iter) data set to the %QCMPRES(
               web) mart failed.;
         %goto ERREXIT;
      %end;

      %************************************************************************;
      %* Append new DETAIL data sets                                           ;
      %************************************************************************;
      data wadetail.detail_&&_wab_update_date_&wab_iter;
         set wadetail.detail_&&_wab_update_date_&wab_iter
             &temp_lib..detail_&&_wab_update_date_&wab_iter;
      run;
      %if &syserr gt 4 %then %do;
         %put %unquote(&wab_error) PROC APPEND for the updated %QCMPRES(
               detail_&&_wab_update_date_&wab_iter) data set to the %QCMPRES(
               web) mart failed.;
         %goto ERREXIT;
      %end;
      data wadetail.detail_tally_&&_wab_update_date_&wab_iter;
         set wadetail.detail_tally_&&_wab_update_date_&wab_iter
             &temp_lib..detail_tally_&&_wab_update_date_&wab_iter;
      run;
      %if &syserr gt 4 %then %do;
         %put %unquote(&wab_error) PROC APPEND for the updated %QCMPRES(
               detail_tally_&&_wab_update_date_&wab_iter) data set to %QCMPRES(
               the) web mart failed.;
         %goto ERREXIT;
      %end;

      %************************************************************************;
      %* Sort the appended DETAIL data set into the standard order.            ;
      %************************************************************************;
      proc sort data=wadetail.detail_&&_wab_update_date_&wab_iter;
         by session_id sequence;
      run;
      %if &syserr gt 4 %then %do;
         %put %unquote(&wab_error) PROC SORT for the updated %QCMPRES(
               detail_&&_wab_update_date_&wab_iter) data set failed.;
         %goto ERREXIT;
      %end;

      %************************************************************************;
      %* Notify that the warehouse has been updated.                           ;
      %************************************************************************;
      %put %unquote(&wab_note) SESSION and DETAIL for %QCMPRES(
                    &&_wab_update_date_&wab_iter) %QCMPRES(
                    have) been appended to earlier entries in the %QCMPRES(
                    warehouse.);
   %end;
%end;


%******************************************************************************;
%* Copy new SESSION/DETAIL data sets for dates that are not yet in the         ;
%*    warehouse.                                                               ;
%******************************************************************************;
%if &_wab_num_new_dates_ gt 0 %then %do;
   proc copy in=&temp_lib out=dated memtype=data;
         select %do wab_iter=1 %to &_wab_num_new_dates_;
                      session_&&_wab_new_date_&wab_iter
                %end;;
   run;
   %if &syserr gt 4 %then %do;
      %put %unquote(&wab_error) PROC COPY for the newly created %QCMPRES(
            session_&&_wab_new_date_&wab_iter) data set to the web mart failed.;
      %goto ERREXIT;
   %end;

   proc copy in=&temp_lib out=wadetail memtype=data;
         select %do wab_iter=1 %to &_wab_num_new_dates_;
                      detail_&&_wab_new_date_&wab_iter
                %end;;
   run;
   %if &syserr gt 4 %then %do;
      %put %unquote(&wab_error) PROC COPY for the newly created %QCMPRES(
            detail_&&_wab_new_date_&wab_iter) data set to the web mart failed.;
      %goto ERREXIT;
   %end;

   proc copy in=&temp_lib out=wadetail memtype=data;
         select %do wab_iter=1 %to &_wab_num_new_dates_;
                      detail_tally_&&_wab_new_date_&wab_iter
                %end;;
   run;
   %if &syserr gt 4 %then %do;
      %put %unquote(&wab_error) PROC COPY for the newly created %QCMPRES(
            detail_tally_&&_wab_new_date_&wab_iter) data set to the %QCMPRES(
            web) mart failed.;
      %goto ERREXIT;
   %end;

   %do wab_iter=1 %to &_wab_num_new_dates_;
      %************************************************************************;
      %* Notify that the warehouse has been added to.                          ;
      %************************************************************************;
      %put %unquote(&wab_note) SESSION and DETAIL for %qcmpres(
                    &&_wab_new_date_&wab_iter) %QCMPRES(
                    have) been added to the warehouse.;
   %end;
%end;


%******************************************************************************;
%* MODULE: II_E - Create Web Analytics Pathing Summary Data                    ;
%* SUPPORT:cabahl                                                              ;
%* Create the top entry paths, paths, and top referrer paths summary data sets ;
%* for 1, 7, and 30 days.                                                      ;
%******************************************************************************;
%let module=II_E;
%let macname=%scan(&macname,1,_)_&module;
%wapathdp(retcode=wab_rc
         );


%******************************************************************************;
%* MODULE: II_F - Data for Interactive Funnels                                 ;
%* SUPPORT:frroed                                                              ;
%* Process Create/maintain the data that supports the REQUESTED_FILE list for  ;
%*    the Interactive Funnel Reports.                                          ;
%******************************************************************************;
%let module=II_F;
%let macname=%scan(&macname,1,_)_&module;

%******************************************************************************;
%*    make sure that SUMMARY.PATHING exists                                    ;
%******************************************************************************;
%if %sysfunc(exist(summary.pathing)) = 0 %then %do;
   %put %unquote(&wab_error) The SUMMARY.PATHING data set does not exist.;
   %put %unquote(&wab_error) DATEs cannot be created for Interactive Funnel.;
   %goto ERREXIT;
%end;

%******************************************************************************;
%*    make sure that SUMMARY.PATHING can be opened.                            ;
%******************************************************************************;
%locktest(dset=summary.pathing);
%if &lock_sw ne 0 %then %do;
   %put %unquote(&wab_error) There was a failure opening SUMMARY.PATHING.;
   %put %unquote(&wab_error) DATEs cannot be created for Interactive Funnel.;
   %goto ERREXIT;
%end;

%******************************************************************************;
%*    make sure that SUMMARY.PATHING has observations.                         ;
%******************************************************************************;
%if %get_observation_count(indsn=summary.pathing)=0  %then %do;
   %put %unquote(&wab_error) SUMMARY.PATHING contains 0 observations.;
   %put %unquote(&wab_error) DATEs cannot be created for Interactive Funnel.;
   %goto ERREXIT;
%end;

%******************************************************************************;
%* Identify the DATEs that the User Interface will allow the User to select    ;
%*    for Interactive Funnels.  Only DATEs that are within the WACONFIG-       ;
%*    specified (via &WAB_PATHING_DAYS_IN_HISTORY) threshhold become part of   ;
%*    this list.                                                               ;
%******************************************************************************;
proc summary data=summary.pathing
             (keep=date) sum chartype;
   class date;
   output out=uniq_dates
            (where=(_type_ eq '1')
            );
run;
%if &syserr gt 4 %then %do;
   %put %unquote(&wab_error) PROC SUMMARY of SUMMARY.PATHING has failed.;
   %put %unquote(&wab_error) DATEs cannot be created for Interactive Funnel.;
   %goto ERREXIT;
%end;
%******************************************************************************;
%*    make sure that UNIQ_DATES has observations.                              ;
%******************************************************************************;
%if %get_observation_count(indsn=uniq_dates)=0  %then %do;
   %put %unquote(&wab_error) UNIQ_DATES contains 0 observations.;
   %put %unquote(&wab_error) DATEs cannot be created for Interactive Funnel.;
   %goto ERREXIT;
%end;


proc sort data=uniq_dates;
   by descending date;
run;

data page_dates;
   attrib date format=date9.;
   set uniq_dates;

   retain _wab_max_page_date_;

   if _n_ eq 1 then do;
      call symput('_wab_max_page_date_',strip(put(date,5.)));
      _wab_max_page_date_=date;
   end;

   if date gt (_wab_max_page_date_
              -&wab_pathing_days_in_history);

   date_c=put(date,date9.);

   keep date
        date_c;

run;

proc sort data=page_dates
          out =summary.page_dates;
   by date;
run;
%if &syserr gt 4 %then %do;
   %put %unquote(&wab_error) PAGE_DATES cannot be written to %QCMPRES(the
      SUMMARY) library.;
   %put %unquote(&wab_error) DATEs cannot be created for Interactive Funnel.;
   %goto ERREXIT;
%end;


%******************************************************************************;
%* MODULE: II_G - Scorecard                                                    ;
%* SUPPORT:sassyw                                                              ;
%* Process the scorecard based on the wascrcrd metadata data set.              ;
%******************************************************************************;
%let module=II_G;
%let macname=%scan(&macname,1,_)_&module;

%wascordp(begin_date=&_wab_min_process_date_
         ,end_date=&_wab_max_process_date_
         ,retcode=wab_rc);

%if &wab_rc ne 0 %then
   %goto ERREXIT;

%******************************************************************************;
%* MODULE: II_H - Dashboard                                                    ;
%* SUPPORT:sassyw                                                              ;
%* Process the dashboard based on the wadshbrd metadata data set.              ;
%******************************************************************************;
%let module=II_H;
%let macname=%scan(&macname,1,_)_&module;

%wadashdp(begin_date=&_wab_min_process_date_
         ,end_date=&_wab_max_process_date_
         ,retcode=wab_rc
         );

%if &wab_rc ne 0 %then
   %goto ERREXIT;


%******************************************************************************;
%* MODULE: II_I - Data for Interactive Pathing                                 ;
%* SUPPORT:frroed                                                              ;
%* Process Create/maintain the data that supports the two REQUESTED_FILE lists ;
%*    (one for path starts, one for path ends) for Interactive Pathing.        ;
%******************************************************************************;
%*    NOTE: this module has been replaced by the %WAPATHDP macro               ;
%******************************************************************************;


%******************************************************************************;
%* MODULE: II_J - Create tallies for Visitors/Repeat Visitors                  ;
%* SUPPORT:cabahl                                                              ;
%* Process Create/maintain SUMMARY data sets with tallies for Visitors/        ;
%*    Repeat Visitors.                                                         ;
%******************************************************************************;
%let module=II_J;
%let macname=%scan(&macname,1,_)_&module;
%wavisitr(min_date=&_wab_min_process_date_
         ,max_date=&_wab_max_process_date_
         ,retcode=wab_rc
         );

%if &wab_rc ne 0 %then
   %goto ERREXIT;


%******************************************************************************;
%* MODULE: II_K - Automatic Segmentation                                       ;
%* SUPPORT:frroed                                                              ;
%* Perform analytic segmentation based on all instances specified in the.      ;
%* config.waautosg data set. The purpose of analytic segmenatation is to relate;
%* a set of input/predictor variables to a target/response variable. Analytic  ;
%* segmentation partitions the input variables in such a way that variation in ;
%* reponse variable is minimized.                                              ;
%******************************************************************************;
%let module=II_K;
%let macname=%scan(&macname,1,_)_&module;

%waautosg(begin_date=&_wab_min_process_date_
         ,end_date=&_wab_max_process_date_
         ,definition_to_run=
         ,retcode=wab_rc
         );

%if &wab_rc ne 0 %then
   %goto ERREXIT;


%******************************************************************************;
%* SECTION: III - Housekeeping                                                 ;
%******************************************************************************;

%******************************************************************************;
%* MODULE: III_A - Age Detail and Session Data sets                            ;
%* SUPPORT:saskxs                                                              ;
%* Remove Detail and Session Data sets older than a certain age range.         ;
%* By default 7 days of detail and 14 days of sessions are stored.             ;                                                    ;
%******************************************************************************;
%let module=III_A;
%let macname=%scan(&macname,1,_)_&module;

proc sort data=&temp_lib..detail_library_contents;
   by descending date ;
run;

data _null_;
   call symput("_wab_detail_count_",'0');
run;

data _null_;
   retain detail_data_count 0;
   set &temp_lib..detail_library_contents;
   detail_data_name=strip(compress(memname));
   if _n_ ge &wab_num_detail_data_sets then do;
      detail_data_count=detail_data_count+1;
      call symput(compress("_wab_detail_name_"
                 ||strip(put(detail_data_count,7.)))
                 ,detail_data_name);
      tally_data_name=tranwrd(upcase(detail_data_name)
                             ,'DETAIL_'
                             ,'DETAIL_TALLY_'
                             );
      call symput(compress("_wab_tally_name_"
                 ||strip(put(detail_data_count,7.)))
                 ,tally_data_name);
   end;

   call symput("_wab_detail_count_",strip(put(detail_data_count,7.)));

run;

%if &_wab_detail_count_ ge 1 %then %do;
   proc datasets library=wadetail nolist;
      delete %do wab_iter=1 %to &_wab_detail_count_;
                 &&_wab_detail_name_&wab_iter
             %end;;
   run;
   quit;
%end;
%if &syserr gt 4 %then %do;
   %put %unquote(&wab_error) Detail data sets in the web mart could %QCMPRES(
        not) be aged.;
   %goto ERREXIT;
%end;

%if &_wab_detail_count_ ge 1 %then %do;
   proc datasets library=wadetail nolist;
      delete %do wab_iter=1 %to &_wab_detail_count_;
                 &&_wab_tally_name_&wab_iter
             %end;;
   run;
   quit;
%end;
%if &syserr gt 4 %then %do;
   %put %unquote(&wab_error) Detail _TALLY data sets in the web mart %QCMPRES(
        could) not be aged.;
   %goto ERREXIT;
%end;

proc sort data=&temp_lib..dated_library_contents;
   by descending date ;
run;

data _null_;
   call symput("_wab_session_count_",'0');
run;


data _null_;
   retain session_data_count 0;
   set &temp_lib..dated_library_contents;
   session_data_name=strip(compress(memname));
   if _n_ ge &wab_num_session_data_sets then do;
      session_data_count=session_data_count+1;
      call symput(compress("_wab_session_name_"
                 ||strip(put(session_data_count,7.)))
                 ,session_data_name);
   end;

   call symput("_wab_session_count_"
              ,strip(put(session_data_count,7.)));

run;

%put &dated_msg;
%if &_wab_session_count_ ge 1 %then %do;
   proc datasets library=dated nolist;
      delete
         %do wab_iter=1 %to &_wab_session_count_;
            &&_wab_session_name_&wab_iter
         %end;;
   quit;
%end;

%if &syserr gt 4 %then %do;
   %put %unquote(&wab_error) Session data sets in the web mart could %QCMPRES(
        not) be aged.;
   %goto ERREXIT;
%end;


%******************************************************************************;
%* MODULE: III_B - Create Updated WAENGVAR                                     ;
%* SUPPORT:frroed                                                              ;
%* Process: Create an updated version of WAENGVAR from the variables in the    ;
%*          DETAIL_yyyymmdd and SESSION_yyyymmdd data sets.                    ;
%******************************************************************************;
%let module=III_B;
%let macname=%scan(&macname,1,_)_&module;
%wametdef(def_dsn=waengvar
          ,retcode=wab_rc
          );

%if &wab_rc ne 0 %then %do;
   %put %unquote(&wab_error) Update failure executing WAMETDEF.;
   %goto ERREXIT;
%end;


%******************************************************************************;
%* MODULE: III_C - Create the _STATUS Reports                                  ;
%* SUPPORT:frroed                                                              ;
%* Process Create/maintain a data set that contains date ranges for Traffic    ;
%*         and Decision Support Reports.                                       ;
%*                                                                             ;
%* NOTE: %WASTATUS is also invoked by Module IV_A.  Logically, either III_C    ;
%*       (normal termination) or IV_A (error exit) will be executed.           ;
%******************************************************************************;
%let module=III_C;
%let macname=%scan(&macname,1,_)_&module;

%wastatus(swamart=&swamart
         ,waetl_sw=Y
         ,retcode=wab_rc
         );

%if &wab_rc ne 0 %then %do;
   %put %unquote(&wab_error) Failure executing WASTATUS.;
   %goto ERREXIT;
%end;


%******************************************************************************;
%* MODULE:  III_D - Generate List Data Sets for UI Selection Lists             ;
%* SUPPORT: frroed                                                             ;
%* Process: Create/maintain data sets that contain selection list entries.     ;
%*                                                                             ;
%* NOTE: %WALISTS is also invoked by Module IV_B.  Logically, either III_D     ;
%*       (normal termination) or IV_B (error exit) will be executed.           ;
%******************************************************************************;
%let module=III_D;
%let macname=%scan(&macname,1,_)_&module;

%walists(retcode=wab_rc
         );

%if &wab_rc ne 0 %then %do;
   %put %unquote(&wab_error) Failure executing WALISTS.;
   %goto ERREXIT;
%end;


%******************************************************************************;
%* Notify that WAETL is completed successfully.                                ;
%******************************************************************************;
%put %unquote(&wab_note) WAETL completed successfully.;


%goto EXIT;


%******************************************************************************;
%* Exit point when fatal errors encountered.                                   ;
%******************************************************************************;
%ERREXIT:


%put %unquote(&wab_error) Web Analytics program exiting due to error.;


%******************************************************************************;
%* SECTION: IV - Post-processing                                               ;
%******************************************************************************;

%******************************************************************************;
%* MODULE: IV_A - Create the _STATUS Reports                                   ;
%* SUPPORT:frroed                                                              ;
%* Process Create/maintain a data set that contains date ranges for Traffic    ;
%*         and Decision Support Reports.                                       ;
%*                                                                             ;
%* NOTE: %WASTATUS is also invoked by Module III_C.  Logically, either III_C   ;
%*       (normal termination) or IV_A (error exit) will be executed.           ;
%******************************************************************************;
%let module=IV_A;
%let macname=%scan(&macname,1,_)_&module;

%wastatus(swamart=&swamart
         ,waetl_sw=Y
         ,retcode=wab_rc
         );

%if &wab_rc ne 0 %then %do;
   %put %unquote(&wab_error) Post-Process failure executing WASTATUS.;
%end;


%******************************************************************************;
%* MODULE:  IV_B - Generate List Data Sets for UI Selection Lists              ;
%* SUPPORT: frroed                                                             ;
%* Process: Create/maintain data sets that contain selection list entries.     ;
%*                                                                             ;
%* NOTE: %WALISTS is also invoked by Module IV_B.  Logically, either III_D     ;
%*       (normal termination) or IV_B (error exit) will be executed.           ;
%******************************************************************************;
%let module=IV_B;
%let macname=%scan(&macname,1,_)_&module;

%walists(retcode=wab_rc
         );

%if &wab_rc ne 0 %then %do;
   %put %unquote(&wab_error) Post-Process failure executing WALISTS.;
%end;


%No_Notes;

%Maybe_Notes;

%* Reset system options to original settings ;
dm 'af catalog=SASHELP.WAGUI.WATRM.SCL';
options COMPRESS=&compress_setting;
options &ovp_setting;
options OBS=&obs_setting;
options &mprint_setting;
options &source2_setting;
options &source_setting;
options &notes_setting;
options &fmterr_setting;
options fmtsearch=&fmtsearch_setting;


%* ABORT/ABEND only if the job is being run in batch -- it terminates          ;
%*    interactive sessions!                                                    ;
%* Interactive sessions have DMS Process, batch has Program 'xxx.sas'          ;
%if not %index(%upcase(&sysprocessname),DMS PROCESS) or
    %index(%upcase(&sysprocessname),.SAS)            or
    %index(%upcase(&sysprocessname),PROGRAM)      %then %do;
   data _null_;
      abort abend 2;
   run;
%end;


%******************************************************************************;
%* Normal exit point.                                                          ;
%******************************************************************************;
%EXIT:
%let module=EXIT;
%let macname=%scan(&macname,1,_)_&module;
%put %unquote(&wab_note) WAETL ending.;


%mend waetl;
